/*
 * Copyright (c) 2010 - 2012 Espressif System.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sys/egl_sys_arch.h"
#include "bus/egl_bus_hw.h"

#include "esp_main_sim.h"
#include "sip2_common.h"

#include "esp_sip.h"
#include "esp_pub.h"
#include "eagle_fw.h"    
//#include "eagle_fw_test1.h"  
#include "linux_inc\list.h"
#include "egl_netbuf.h"
#include "esp_utils.h"


#define SIF_TO_DEVICE                    0x1
#define SIF_FROM_DEVICE                  0x2

#define SIF_SYNC            0x00000010
#define SIF_ASYNC           0x00000020

#define SIF_BYTE_BASIS              0x00000040
#define SIF_BLOCK_BASIS             0x00000080

#define SIF_FIXED_ADDR   0x00000100
#define SIF_INC_ADDR     0x00000200

#define SIF_SLC_BLOCK_SIZE                512
#define REG_SLC_HOST_BASE  0x00000000
#define SLC_HOST_DATE                         (REG_SLC_HOST_BASE + 0x78)
#define SLC_HOST_ID                           (REG_SLC_HOST_BASE + 0x7C)

#define ESP_DMA_IBUFSZ   2048

void *EpubAddr;
char GlBootupCplx = 0;

static inline unsigned char bad_buf(u8 * buf)
{
        //return ((unsigned long) buf & 0x3);
        return false;
}


void check_target_id(void *psctrl);
int esp_pub_init_all(struct esp_pub *epub);
struct esp_sip * sip_attach(struct esp_pub *epub) ;

int sip_prepare_boot(struct esp_sip *sip);
int esp_download_fw(struct esp_pub * epub);
int sip_write_memory(struct esp_sip *sip, u32 addr, const u8 *buf, u16 len);
int sif_lldesc_write_sync(struct esp_pub *epub, u8 *buf, u32 len);
int sif_io_sync(struct esp_pub *epub, u32 addr, u8 *buf, u32 len, u32 flag);

int sip_send_cmd(struct esp_sip *sip, int cid, u32 cmdlen, void *cmd);
static struct sip_pkt *sip_get_ctrl_buf(struct esp_sip *sip, SIP_BUF_TYPE bftype) ;
static void sip_reclaim_ctrl_buf(struct esp_sip *sip, struct sip_pkt *pkt, SIP_BUF_TYPE bftype);
static void sip_dec_credit(struct esp_sip *sip);

void sif_enable_irq(struct esp_pub *epub); 
int sip_poll_bootup_event(struct esp_sip *sip);


void sif_dsr(void);
void esp_dsr(struct esp_pub *epub);

    
int esp_sdio_probe(void) 
{
        int err;
        struct esp_pub *epub;
        struct esp_sdio_ctrl *sctrl;

        sctrl = ( struct esp_sdio_ctrl * )egl_malloc( sizeof(struct esp_sdio_ctrl) );

        if (sctrl == NULL) 
        {
                printf("error, esp_sdio_ctrl no memory\n");
        }
        
        epub = esp_pub_alloc_mac80211();

        if (epub == NULL) 
        {
                printf("error, epub no memory\n");
        }
        
        EpubAddr = epub;

        epub->sif = (void *)sctrl;
        sctrl->epub = epub;

        check_target_id(sctrl);

        sctrl->slc_blk_sz = SIF_SLC_BLOCK_SIZE;

        err = esp_pub_init_all(epub);

        return err;
}

void check_target_id(void *psctrl)
{
        struct esp_sdio_ctrl *sctrl = (struct esp_sdio_ctrl *)psctrl;
        u32 date;
        int err = 0;
        int i;

        for(i = 0; i < 4; i++) 
        {
                *((u8 *)&date + i) = BusReadReg( SLC_HOST_DATE + i, &err);
                *((u8 *)&sctrl->target_id + i) = BusReadReg( SLC_HOST_ID + i, &err);
        }

        printf("\t\t SLC data 0x%08x, ID 0x%08x\n\n", date, sctrl->target_id); 
       
        switch(sctrl->target_id) 
        {
                case 0x100:
                sctrl->slc_window_end_addr = 0x20000;
                break;
                case 0x600:
                sctrl->slc_window_end_addr = 0x20000 - 0x800;
                break;
        default:
                sctrl->slc_window_end_addr = 0x20000;
                break;
        }
}

int esp_pub_init_all(struct esp_pub *epub)
{
  int ret = 0;
  epub->sip = sip_attach(epub);          
  ret = esp_download_fw(epub);
  GlBootupCplx = 0;
  epub->sip->state=SIP_BOOT;
  return ret;
}

struct esp_sip * sip_attach(struct esp_pub *epub) 
{
        struct esp_sip *sip = NULL;
        struct sip_pkt *pkt = NULL;
        int i;

        sip = (struct esp_sip *)egl_malloc(sizeof(struct esp_sip));
        
        egl_mutex_new(&sip->lock);
    
        INIT_LIST_HEAD(&sip->free_ctrl_txbuf);
        INIT_LIST_HEAD(&sip->free_ctrl_rxbuf);

#if 1
        for (i = 0; i < SIP_CTRL_BUF_N; i++) 
        {
                pkt = (struct sip_pkt *)egl_malloc( sizeof(struct sip_pkt) );
                pkt->buf_begin =(u8* )egl_malloc( SIP_CTRL_BUF_SZ );

                if (pkt->buf_begin == NULL) 
                {
                        egl_free(pkt);
                        break;
                }
                
                pkt->buf_len = SIP_CTRL_BUF_SZ;
                pkt->buf = pkt->buf_begin;
                
                if (i < SIP_CTRL_TXBUF_N) 
                {
                        list_add_tail(&pkt->list, &sip->free_ctrl_txbuf);
                } 
                else 
                {
                        list_add_tail(&pkt->list, &sip->free_ctrl_rxbuf);
                }
        }
#endif
        egl_mutex_new( &(sip->rx_lock) );
        A_NETBUF_QUEUE_INIT(&sip->rxq);
        sip->epub = epub;
        sip->state = SIP_INIT;
        return sip;
}


struct esp_fw_hdr {
        u8 magic;
        u8 blocks;
        u8 pad[2];
        u32 entry_addr;
} __packed;

struct esp_fw_blk_hdr {
        u32 load_addr;
        u32 data_len;
} __packed;

#define ESP_FW_NAME "eagle_fw.bin"


int esp_download_fw(struct esp_pub * epub)
{

        const unsigned char * fw_buf = NULL;
        unsigned int offset = 0;
        int ret = 0;
        unsigned char blocks;
        struct esp_fw_hdr *fhdr;
        struct esp_fw_blk_hdr *bhdr=NULL;
        struct sip_cmd_bootup bootcmd;

        fw_buf = &eagle_fw[0];

        fhdr = (struct esp_fw_hdr *)fw_buf;

        if (fhdr->magic != 0xE9) {
                printf( "wrong magic! \n");
                goto _err;
        }

        ret = sip_prepare_boot(epub->sip);

        blocks = fhdr->blocks;
        offset += sizeof(struct esp_fw_hdr);
        
        while (blocks) 
        {
                bhdr = (struct esp_fw_blk_hdr *)(&fw_buf[offset]);
                offset += sizeof(struct esp_fw_blk_hdr);

                printf("blocks:%x ->%x,%x\n",blocks,bhdr->load_addr,bhdr->data_len);

                ret = sip_write_memory(epub->sip, bhdr->load_addr, &fw_buf[offset], bhdr->data_len);

                blocks--;
                offset += bhdr->data_len;
        }
		// write boot info
		// ret = sip_write_memory(epub->sip, 0x4010F000, (const u8*)fhdr, sizeof(struct esp_fw_hdr *));

        /* TODO: last byte should be the checksum and skip checksum for now */

        bootcmd.boot_addr = fhdr->entry_addr;
        ret = sip_send_cmd(epub->sip, SIP_CMD_BOOTUP, sizeof(struct sip_cmd_bootup), &bootcmd);

        if (ret)
                goto _err;

_err:
        return ret;
        
}



int sip_prepare_boot(struct esp_sip *sip)
{
        if (sip->state != SIP_INIT)
        {
                //esp_dbg(ESP_DBG_ERROR, "%s wrong state %d\n", __func__, atomic_read(&sip->state));
                return -1;
        }

        sip->rawbuf = NULL;
        if (sip->rawbuf == NULL) 
        {
                //printf("sip rawbuf memory malloc \n");
                sip->rawbuf = (unsigned char *)egl_malloc(SIP_BOOT_BUF_SIZE);
                if (sip->rawbuf == NULL)
                {
                        printf("sip rawbuf memory not enough\n");
                        return -1;
                 }
        }

        sip->state = SIP_PREPARE_BOOT;
        
	return 0;
}



int sip_write_memory(struct esp_sip *sip, u32 addr, const u8 *buf, u16 len)
{
        struct sip_cmd_write_memory *cmd;
        struct sip_hdr *chdr;
        u16 remains, hdrs, bufsize;
        u32 loadaddr;
        const u8 *src;
        int err = 0;
        u32 *t = 0;
        
        //ASSERT(sip->rawbuf != NULL);

        memset(sip->rawbuf, 0, SIP_BOOT_BUF_SIZE);
        
        chdr = (struct sip_hdr *)sip->rawbuf;
        SIP_HDR_SET_TYPE(chdr->fc[0], SIP_CTRL);
        chdr->c_cmdid = SIP_CMD_WRITE_MEMORY;
        
        remains = len;
        hdrs = sizeof(struct sip_hdr) + sizeof(struct sip_cmd_write_memory);
        
        while (remains) 
        {
                src = &buf[len - remains];
                loadaddr = addr + (len - remains);

                if (remains < (SIP_BOOT_BUF_SIZE - hdrs)) 
                {
                        /* aligned with 4 bytes */
                        bufsize = roundup(remains, 4);
                        memset(sip->rawbuf + hdrs, 0, bufsize);
                        remains = 0;
                        
                } else {
                        bufsize = SIP_BOOT_BUF_SIZE - hdrs;
                        remains -=  bufsize;
                }

                chdr->len = bufsize + hdrs ;
                chdr->seq = sip->txseq++;
                cmd = (struct sip_cmd_write_memory *)(sip->rawbuf + SIP_CTRL_HDR_LEN);
                cmd->len = bufsize;
                cmd->addr = loadaddr;
                
                memcpy(sip->rawbuf+hdrs, src, bufsize);
                t = (u32 *)sip->rawbuf;
        
                err = sif_lldesc_write_sync(sip->epub, sip->rawbuf, chdr->len);

                if (err) {
                        //esp_dbg(ESP_DBG_ERROR, "%s send buffer failed\n", __func__);
                        printf("Send buffer failed\n");
                        return err;
                }

                // 1ms is enough, in fact on dell-d430, need not delay at all.
                egl_delay(1);
                sip_dec_credit(sip);
        }

        return err;
}


int sif_lldesc_write_sync(struct esp_pub *epub, u8 *buf, u32 len)
{
        struct esp_sdio_ctrl *sctrl = NULL;
        u32 write_len;

       // ASSERT(epub != NULL);

        sctrl = (struct esp_sdio_ctrl *)epub->sif;

        switch(sctrl->target_id) {
        case 0x100:
                write_len = len;
                break;
        case 0x600:
                write_len = roundup(len, sctrl->slc_blk_sz);
                write_len = write_len/(sctrl->slc_blk_sz);
                //write_len = sctrl->slc_blk_sz;
                break;
        default:
                write_len = len;
                break;
        }

        // For larger than 512 bytes, splite read bytes. 
        return sif_io_sync((epub), (sctrl->slc_window_end_addr - (len)), (buf), (write_len), SIF_TO_DEVICE | SIF_BYTE_BASIS | SIF_INC_ADDR);
        //return 0;
}


u32 sif_get_blksz(struct esp_pub *epub)
{
        struct esp_sdio_ctrl *sctrl = NULL;
        sctrl = (struct esp_sdio_ctrl *)epub->sif;
        return sctrl->slc_blk_sz;
}

int sif_lldesc_read_sync(struct esp_pub *epub, u8 *buf, u32 len)
{
        struct esp_sdio_ctrl *sctrl = NULL;
        u32 read_len;

        //ASSERT(epub != NULL);

        sctrl = (struct esp_sdio_ctrl *)epub->sif;

        switch(sctrl->target_id) {
        case 0x100:
                read_len = len;
                break;
        case 0x600:
                read_len = roundup(len, sctrl->slc_blk_sz);
                read_len = read_len/(sctrl->slc_blk_sz);
                //read_len = sctrl->slc_blk_sz;
                break;
        default:
                read_len = len;
                break;
        }
        //return sif_io_sync((epub), (sctrl->slc_window_end_addr - 2 - (len)), (buf), (read_len), SIF_FROM_DEVICE | SIF_BYTE_BASIS | SIF_INC_ADDR);
        return sif_io_sync((epub), (sctrl->slc_window_end_addr - 2 - (len)), (buf), (read_len), SIF_FROM_DEVICE | SIF_BLOCK_BASIS | SIF_INC_ADDR);
}


int sif_io_sync(struct esp_pub *epub, u32 addr, u8 *buf, u32 len, u32 flag)
{

//printf("addr:%X,len:%X \n",addr,len);

        int err = 0;
        u8 * ibuf = NULL;
   
        ibuf = buf;

        if (flag & SIF_BLOCK_BASIS) {
                /* round up for block data transcation */
        }

        if (flag & SIF_TO_DEVICE) 
        {

                bus_claim_host();

                if (flag & SIF_FIXED_ADDR)   ;
                else if (flag & SIF_INC_ADDR) 
                {
//                        err =sif_spi_write_blocks(addr, ibuf, len,SPI_BLOCK_SIZE);
                        err = BusWriteBlocks( addr, ibuf, len);
                }

                bus_release_host();
        }
        else if (flag & SIF_FROM_DEVICE) 
        {
                bus_claim_host();
                if (flag & SIF_FIXED_ADDR);
                else if (flag & SIF_INC_ADDR)
                {
//                    err =sif_spi_read_blocks(addr, ibuf, len,SPI_BLOCK_SIZE);
                    err = BusReadBlocks(addr, ibuf, len);
                }

                bus_release_host();
        }

        return err;
}



//sip_send_cmd(epub->sip, SIP_CMD_BOOTUP, sizeof(struct sip_cmd_bootup), &bootcmd);
int sip_send_cmd(struct esp_sip *sip, int cid, u32 cmdlen, void *cmd)
{
        struct sip_hdr *chdr;
        struct sip_pkt *pkt = NULL;
        int ret = 0;
        
        pkt = sip_get_ctrl_buf(sip, SIP_TX_CTRL_BUF);

        if (pkt == NULL)
                return -1;

        chdr = (struct sip_hdr *)pkt->buf_begin;
        chdr->len = SIP_CTRL_HDR_LEN + cmdlen; 
        
//printf("1 sip->txseq:%d\n",sip->txseq);
 //if( cid !=SIP_CMD_INIT)
        chdr->seq = sip->txseq++;
        chdr->c_cmdid = cid;
	
	if (cmd) 
       {
		memset(pkt->buf, 0, cmdlen);
		memcpy(pkt->buf, (u8 *)cmd, cmdlen);
	}

        //esp_dbg(ESP_DBG_TRACE, "cid %d, len %u, seq %u \n", chdr->c_cmdid, chdr->len, chdr->seq);
        //esp_dbg(ESP_DBG_TRACE, "c1 0x%08x   c2 0x%08x\n", *(u32 *)&pkt->buf[0], *(u32 *)&pkt->buf[4]);

        ret = sif_lldesc_write_sync(sip->epub, pkt->buf_begin, chdr->len);

        if (ret)
                printf("send cmd %d failed \n", cid);

        sip_dec_credit(sip);

        sip_reclaim_ctrl_buf(sip, pkt, SIP_TX_CTRL_BUF);

        /*
         *  Hack here: reset tx/rx seq before target ram code is up...
         */
        if (cid == SIP_CMD_BOOTUP) {
                sip->rxseq = 0;
                sip->txseq = 0;
 //printf("2 sip->txseq:%d\n",sip->txseq);
        }
        return ret;
}

static struct sip_pkt *sip_get_ctrl_buf(struct esp_sip *sip, SIP_BUF_TYPE bftype) 
{
        struct sip_pkt *pkt = NULL;
        struct list_head *bflist;
        struct sip_hdr *chdr;

        bflist = (bftype == SIP_TX_CTRL_BUF) ? &sip->free_ctrl_txbuf :&sip->free_ctrl_rxbuf;

        //spin_lock_bh(&sip->lock);
        egl_mutex_lock(&sip->lock);

        if (list_empty(bflist)) {
                //spin_unlock_bh(&sip->lock);
                egl_mutex_unlock(&sip->lock);
                return NULL;
        }

        pkt = list_first_entry(bflist, struct sip_pkt, list);
        list_del(&pkt->list);
        //spin_unlock_bh(&sip->lock);
        egl_mutex_unlock(&sip->lock);

        if (bftype == SIP_TX_CTRL_BUF) {
                chdr = (struct sip_hdr *)pkt->buf_begin;
                SIP_HDR_SET_TYPE(chdr->fc[0], SIP_CTRL);
                pkt->buf = pkt->buf_begin + SIP_CTRL_HDR_LEN;
        } else {
                pkt->buf = pkt->buf_begin;
        }

        return pkt;
}


int sip_poll_bootup_event(struct esp_sip *sip)
{
	int ret = 0;

        //esp_dbg(ESP_DBG_TRACE, "polling bootup event... \n");

	if (GlBootupCplx == 0)
            return 0;
       else
       {
                sip->state = SIP_RUN;
                printf("target booted up\n");
                return 1;
       }
       
	return ret;
}

int SipPollBootupEvent(void)
{
        return sip_poll_bootup_event( ((struct esp_pub *)EpubAddr)->sip);
}


static void sip_reclaim_ctrl_buf(struct esp_sip *sip, struct sip_pkt *pkt, SIP_BUF_TYPE bftype)
{
        struct list_head *bflist = NULL;

        if (bftype == SIP_TX_CTRL_BUF)
                bflist = &sip->free_ctrl_txbuf;
        else if (bftype == SIP_RX_CTRL_BUF)
                bflist = &sip->free_ctrl_rxbuf;
        else return;

        pkt->buf = pkt->buf_begin;
        pkt->payload_len = 0;

        //spin_lock_bh(&sip->lock);
        egl_mutex_lock(&sip->lock);
        
        list_add_tail(&pkt->list, bflist);
        
        //spin_unlock_bh(&sip->lock);
        egl_mutex_unlock(&sip->lock);
}


static void sip_dec_credit(struct esp_sip *sip)
{
        sip->boot_credits--;
}


/*    Below are ISQ codes        */
/********жݽ*****/

void esp_dsr(struct esp_pub *epub)
{
        egl_mutex_lock( &(epub->sip->rx_lock) );
        
        sip_rx(epub);
        
        egl_mutex_unlock( &(epub->sip->rx_lock) );
}

void sif_dsr(void)
{
        esp_dsr((struct esp_pub *)EpubAddr);
}


/*    Below are skb buff process codes        */
/*********************/
void SipDataProcess(void)
{
        sip_rxq_process( (struct esp_pub *)EpubAddr );
}



/*    Below are skb buff process codes        */
/*********************/

void esp_tx_work(void)
{
        struct esp_pub *epub = (struct esp_pub *)EpubAddr ;

/*
        static int check = 0;
        check++;
        printf("s:%d\n",check);
*/     
        egl_mutex_lock( &(epub->tx_mtx) );
        sip_txq_process(epub);
        egl_mutex_unlock( &(epub->tx_mtx) );
}


