/*
 * Copyright (c) 2011-2012 Espressif System.
 *
 *   wlan device header file
 */

#ifndef __ESP_PUB_H__
#define __ESP_PUB_H__


#include "sys/egl_sys_arch.h"
#include "sip2_common.h"
#include "esp_sip.h"

#include "linux_inc\net\ieee80211.h"
#include "linux_inc\net\cfg80211.h"
#include "linux_inc\net\mac80211.h"

#include "egl_netbuf.h"

enum esp_tid_state {
        ESP_TID_STATE_INIT,
        ESP_TID_STATE_TRIGGER,
        ESP_TID_STATE_PROGRESS,
        ESP_TID_STATE_OPERATIONAL,
        ESP_TID_STATE_WAIT_STOP,
        ESP_TID_STATE_STOP,
};

struct esp_tx_tid {
        u8 state;
        u16 ssn;
};

#if 1
#define WME_NUM_TID 16
struct esp_node {
        struct esp_tx_tid tid[WME_NUM_TID];
        struct ieee80211_sta *sta;
	u8 ifidx;
	u8 index;
};
#endif

#define WME_AC_BE 2
#define WME_AC_BK 3
#define WME_AC_VI 1
#define WME_AC_VO 0

struct llc_snap_hdr {
        u8 dsap;
        u8 ssap;
        u8 cntl;
        u8 org_code[3];
        //__be16 eth_type;
        u16 eth_type;
} __packed;


struct timer_list {
	/*
	 * All fields that change during normal runtime grouped to the
	 * same cacheline
	 */
	unsigned long expires;
	
	egl_timer_fn function;
	unsigned long data;
	egl_timer_t timer;
};

struct esp_vif {
	struct esp_pub *epub;
	u8 index;
	u32 beacon_interval;
	bool ap_up;
	struct timer_list beacon_timer;
};

/* WLAN related, mostly... */
/*
struct hw_scan_timeout {
        struct delayed_work w;
        struct ieee80211_hw *hw;
};
*/

typedef struct esp_wl {
        u8 ssid[IEEE80211_MAX_SSID_LEN];
        u8 dot11_auth_mode;
        u8 auth_mode;
        u8 prwise_crypto;
        u8 prwise_crypto_len;
        u8 grp_crypto;
        u8 grp_crpto_len;
        u8 def_txkey_index;
        u32 used_key_entries;   //each bit represent an entry, 0: avail 1: used

        u8 bssid[ETH_ALEN];
        u8 req_bssid[ETH_ALEN];
        u16 ch_hint;
        u16 bss_ch;
        u16 listen_intvl_b;
        u16 listen_intvl_t;
        u32 beacon_interval;

        //struct ieee80211_vif * vif;

        //struct hw_scan_timeout *hsd;
        struct cfg80211_scan_request *scan_req;
		//atomic_t ptk_cnt;
		//atomic_t gtk_cnt;
        u32 ptk_cnt;
        u32 gtk_cnt;
		
        //struct ieee80211_key_conf *ptk;
        //struct ieee80211_key_conf *gtk[4];

        //struct list_head amsdu_rx_buffer_queue;
        //struct timer_list disconnect_timer;
        //struct timer_list beacon_timer;

        /* so far only 2G band */
        //struct ieee80211_supported_band sbands[IEEE80211_NUM_BANDS];
        //enum nl80211_iftype type; //TBD: support multiple mode concurrently
        bool ap_up;

        unsigned long flags;
        u8 off;
} esp_wl_t;


#define ESP_TXQ_MAX_LEN 64 /* TBD: better #? */
#define ESP_PUB_F_TXPAUSE  0x1

#define ESP_WL_FLAG_RFKILL                	BIT(0)
#define ESP_WL_FLAG_HW_REGISTERED   		BIT(1)
#define ESP_WL_FLAG_CONNECT              		BIT(2)
#define ESP_WL_FLAG_STOP_TXQ          		BIT(3)

#define ESP_PUB_MAX_VIF		2
#define ESP_PUB_MAX_STA		4 //for one interface

struct esp_pub {

        struct ieee80211_conf conf;
        struct ieee80211_vif *vif;
        u8 vif_slot;

        void *sif; /* serial interface control block, e.g. sdio */
        struct esp_sip *sip;
        struct esp_wl wl;

        u32 flags; //flags to represent rfkill switch,start
        u8 roc_flags;   //0: not in remain on channel state, 1: in roc state

        void *p_netif;   // upper layer netif point.
        egl_mbox_t *user_evt_mbox;
        egl_mutex_t rx_lock;
        A_NETBUF_QUEUE_T rxq;

        egl_mutex_t tx_mtx;
        A_NETBUF_QUEUE_T txq;
        A_NETBUF_QUEUE_T txdoneq;
        bool txq_stopped;

        u8 bssid[ETH_ALEN];
        u8 mac_addr[ETH_ALEN];

        u32 rx_filter;
        unsigned long scan_permit;
        bool scan_permit_valid;

        u8 enodes_map;
        u8 enodes_maps[ESP_PUB_MAX_VIF];
        struct esp_node * enodes[ESP_PUB_MAX_STA + 1];
};


int esp_op_start(void);
void esp_op_stop(void);

int esp_op_sta_add(struct ieee80211_vif *vif, struct ieee80211_sta *sta);
int esp_op_sta_remove(struct ieee80211_vif *vif, struct ieee80211_sta *sta);

int esp_op_tx( struct sk_buff *skb);
int esp_ini_user_para(void *p_para, int size, u8 flag);
struct esp_pub *esp_pub_alloc_mac80211(void);


#endif /* _ESP_PUB_H_ */
