/*
 * Copyright (c) 2009 - 2013 Espressif System.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sys/egl_sys_arch.h"

#include "linux_inc\net\cfg80211.h"
#include "linux_inc\net\ieee80211.h"
#include "linux_inc\net\mac80211.h"

#include "sip2_common.h"
#include "esp_main_sim.h"
#include "egl_netbuf.h"
#include "esp_pub.h"
#include "esp_ctrl.h"
#include "esp_wmac.h"
#include "esp_utils.h"
#include "port/wifi_user_interface.h"
#include "esp_debug.h"

#define SIP_STOP_QUEUE_THRESHOLD 32
#define SIP_RESUME_QUEUE_THRESHOLD  16

#define SIP_PENDING_STOP_TX_THRESHOLD 6
#define SIP_PENDING_RESUME_TX_THRESHOLD 6
#define SIP_MIN_DATA_PKT_LEN    (sizeof(struct esp_mac_rx_ctrl) + 24) //24 is min 80211hdr

int RXQueueWorkFlag;
int TXQueueWorkFlag;
egl_sem_t QueueWorkCntSem;

static void sip_trigger_txq_process(struct esp_sip *sip);

//static inline void sip_rx_pkt_enqueue(struct esp_sip *sip, struct sk_buff *skb)
static inline void sip_rx_pkt_enqueue(struct esp_sip *sip, void *skb_pkt)
{
        //skb_queue_tail(&sip->rxq, skb);
        A_NETBUF_ENQUEUE_TAIL(&sip->rxq, (skb_t *)skb_pkt);
}

/***********жݽ*********************/
extern void *Memcpy(void *dest, const void *src, size_t count);
int sip_rx(struct esp_pub *epub)
{
        struct sip_hdr *shdr = NULL;
        struct esp_sip *sip = epub->sip;
        int err = 0;
        
        //struct sk_buff *first_skb = NULL;
        //struct sk_buff *next_skb = NULL;
        void *first_skb = NULL;
        void *next_skb = NULL;
        
        u8 *rx_buf = NULL;
        u32 rx_blksz;

        if (sip->state == SIP_STOP)
        {
                printf("when sip stopped \n");
                return 0;
        }
        
        /* first read one block out, if we luck enough, that's it
         *
         *  To make design as simple as possible, we allocate skb(s)
         *  separately for each sif read operation to avoid global
         *  read_buf_pointe access.  It coule be optimized late.
         */
        rx_blksz = sif_get_blksz(epub);
        //printf("rx_blksz =%d\n",rx_blksz );
      
        //first_skb = __dev_alloc_skb(rx_blksz, GFP_KERNEL);
        first_skb = A_NETBUF_ALLOC(rx_blksz);
        
        if (first_skb == NULL) 
        {
                //esp_sip_dbg(ESP_DBG_ERROR, "%s first no memory \n", __func__);
                printf("first no memory \n");
                goto _err;
        }

        rx_buf = A_NETBUF_DATA(first_skb);
        A_NETBUF_PUT(first_skb, rx_blksz);
        
        //rx_buf = skb_put(first_skb, rx_blksz);
        //esp_sip_dbg(ESP_DBG_LOG, "%s rx_buf ptr %p, rx_blksz %d\n", __func__, rx_buf, rx_blksz);
/*
if( bad_buf(rx_buf) )
       memset(rx_buf,0xFF,10);
   */   
//printf("rxbuf addr: %x\n",rx_buf);
        err = sif_lldesc_read_sync(epub, rx_buf, rx_blksz);
//printf("r\n");

        if (err != 0) 
        {
                //esp_dbg(ESP_DBG_ERROR, " %s first read err %d %d\n", __func__, err, sif_get_regs(epub)->config_w0);
                printf("first read err \n");
                A_NETBUF_FREE(first_skb);
                goto _err;
        }

#if 0
        for(char i=0;i<20;i++)
                printf("%x,",rx_buf[i]);
        printf("\n");  
#endif 

        shdr = (struct sip_hdr *)rx_buf;
//printf("shdr->fc[0]: %d, evid: 0x%x, len: %d,credits: %x, seq:%d \n",shdr->fc[0],shdr->fc[1],shdr->len,shdr->u.recycled_credits, shdr->seq);
//printf("shdr->fc[0]:%d, evid: 0x%x, len =%d\n",shdr->fc[0], shdr->fc[1], shdr->len );
        //ASSERT((shdr->len & 3) == 0);
        //ASSERT(shdr->len >= SIP_HDR_LEN && shdr->len <= SIP_PKT_MAX_LEN);

        if (shdr->len > rx_blksz)  
        {
               // printf("shdr->len > rx_blksz \n");
#if 1
                /* larger than one blk, fetch the rest */
                //next_skb = __dev_alloc_skb(roundup(shdr->len, rx_blksz), GFP_KERNEL);
                next_skb = A_NETBUF_ALLOC( roundup(shdr->len, rx_blksz) );

                if (next_skb == NULL)
                {
                        //sif_unlock_bus(epub);
                        //esp_sip_dbg(ESP_DBG_ERROR, "%s next no memory \n", __func__);
                        printf("no memory for next_skb\n");
                        A_NETBUF_FREE(first_skb);
                        goto _err;
                }
                
                //rx_buf = skb_put(next_skb, shdr->len);
                rx_buf = A_NETBUF_DATA(next_skb);
                A_NETBUF_PUT(next_skb, shdr->len);
                    
                rx_buf += rx_blksz; /* skip the first block */

                err = sif_lldesc_read_sync(epub, rx_buf, (shdr->len - rx_blksz));

                if (err)
                {
                        //esp_sip_dbg(ESP_DBG_ERROR, "%s next read err %d \n", __func__, err);
                        printf("next read err %d \n");
                        A_NETBUF_FREE(first_skb);
                        A_NETBUF_FREE(next_skb);
                        goto _err;
                }
                
                /* merge two skbs, TBD: could be optimized by skb_linearize*/
                Memcpy( ((skb_t*)next_skb)->data,((skb_t*) first_skb)->data, rx_blksz);
                //esp_dbg(ESP_DBG_TRACE, " %s  next skb\n", __func__);

                A_NETBUF_SETLEN(next_skb, shdr->len);
                sip_rx_pkt_enqueue(sip, next_skb);
                A_NETBUF_FREE(first_skb);
#endif
        } 
        else 
        {
                A_NETBUF_SETLEN(first_skb, shdr->len);
                //skb_trim(first_skb, shdr->len);
                //esp_dbg(ESP_DBG_TRACE, " %s first_skb only\n", __func__);
                sip_rx_pkt_enqueue(sip, first_skb);
        }
        RXQueueWorkFlag++;
        //egl_sem_signal(&QueueWorkCntSem);
        //printf("RXQ Flag:%d\n",RXQueueWorkFlag);
_err:
        return err;
}







/***********ݵĴ*********************/

//static struct sk_buff * sip_parse_data_rx_info(struct esp_sip *sip, struct sk_buff *skb, int pkt_len_enc, int buf_len, struct esp_mac_rx_ctrl *mac_ctrl, int *pulled_len) 
static struct sk_buff * sip_parse_data_rx_info(struct esp_sip *sip, struct sk_buff *skb)
{
        struct sip_hdr *hdr = (struct sip_hdr *)skb->data;
        struct sk_buff *rskb = NULL;
        short rskb_data_len,pull_out_data_len;

        pull_out_data_len = SIP_HDR_LEN + SIP_PAYLOAD_RCC_LEN + hdr->fc[1];
        rskb_data_len = hdr->len - pull_out_data_len ;

        if(rskb_data_len<0)
        {
                printf("skb data_len < 0 \n");
                return NULL;
        }
        
        do 
        {
                rskb = A_NETBUF_ALLOC( rskb_data_len );
                if (rskb == NULL) 
                {
                        printf("no mem for rskb\n");
                        return NULL;
                }
                A_NETBUF_PUT(rskb, rskb_data_len);
                        
         } while(0);

        A_NETBUF_PULL(skb, pull_out_data_len);        
        do 
        {
                memcpy( rskb->data, skb->data, rskb_data_len );
                /* strip out current pkt, move to the next one */
                //skb_pull(skb, buf_len);
                A_NETBUF_PULL(skb, rskb_data_len);
        } while (0);

        //esp_dbg(ESP_DBG_LOG, "%s after pull headers, skb->len %d rskb->len %d \n", __func__, skb->len, rskb->len);
//printf("after pull headers, skb->len %d rskb->len %d \n",skb->len, rskb->len);

        return rskb;
}


static u8 sip_rx_pkt_process(struct esp_sip * sip, struct sk_buff *skb)
{
	struct sip_hdr * hdr = NULL;
	struct sk_buff * rskb = NULL;
	int remains_len = 0;
	int first_pkt_len = 0;
	u8 *bufptr = NULL;
	int ret = 0;
	u8 trigger_rxq = false;
    
	if (skb == NULL)
       {
		//esp_sip_dbg(ESP_DBG_ERROR, "%s NULL SKB!!!!!!!! \n", __func__);
		return trigger_rxq;
	}

	hdr = (struct sip_hdr *)skb->data;
	bufptr = skb->data;

	//esp_sip_dbg(ESP_DBG_TRACE, "%s Hcredits 0x%08x, realCredits %d\n", __func__, hdr->h_credits, hdr->h_credits & SIP_CREDITS_MASK);

	if (hdr->h_credits & SIP_CREDITS_MASK) 
       {
		//sip_update_tx_credits(sip, hdr->h_credits & SIP_CREDITS_MASK);
		//  printf("before add, credits is %x\n", sip->tx_credits );
                sip->tx_credits += (hdr->h_credits & SIP_CREDITS_MASK);
             //   printf("after add %x, credits is %x\n", hdr->h_credits & SIP_CREDITS_MASK, sip->tx_credits );
	}

	hdr->h_credits &= ~SIP_CREDITS_MASK; /* clean credits in sip_hdr, prevent over-add */

	//esp_sip_dbg(ESP_DBG_TRACE, "%s credits %d\n", __func__, hdr->h_credits);

	/*
	 * first pkt's length is stored in  recycled_credits first 20 bits
	 * config w3 [31:12]
	 * repair hdr->len of first pkt
	 */
	remains_len = hdr->len;
	first_pkt_len = hdr->h_credits >> 12;
	hdr->len = first_pkt_len;

	//esp_dbg(ESP_DBG_TRACE, "%s first_pkt_len %d, whole pkt len %d \n", __func__, first_pkt_len, remains_len);

	if (first_pkt_len > remains_len) 
        {
		printf("err, first_pkt_len %d, whole pkt len %d\n", first_pkt_len, remains_len);
		//show_buf((u8 *)hdr, first_pkt_len);
		//ASSERT(0);
	}

	/*
	 * pkts handling, including the first pkt, should alloc new skb for each data pkt.
	 * free the original whole skb after parsing is done.
	 */
	 
	while (remains_len)
       {
       
//printf("remains_len: %d, hdr->len: %d \n", remains_len, hdr->len);
		//ASSERT(remains_len >= sizeof(struct sip_hdr));
		hdr = (struct sip_hdr *)bufptr;
//printf("hdr->fc[0]:%d, evid: 0x%x, len =%d\n",hdr->fc[0], hdr->fc[1], hdr->len );

		//ASSERT(hdr->len > 0);
		
//pkt len  Must roundup to 4. 
		if((hdr->len & 3) != 0) 
              {
                    //printf("hdr->len &3 !=0 \n");
			//show_buf((u8 *)hdr, 512);
		}
		//ASSERT((hdr->len & 3) == 0);
		
		/*
		if (unlikely(hdr->seq != sip->rxseq++)) {
			esp_dbg(ESP_DBG_ERROR, "%s seq mismatch! got %u, expect %u\n", __func__, hdr->seq, sip->rxseq-1);
			show_buf(bufptr, 32);
			ASSERT(0);
		}
              */
              
//if(!(hdr->seq%100))
  //  printf("rx_seq:%u\n",(hdr->seq/100));
              
		if (hdr->seq != sip->rxseq++) 
              {
			printf("seq mismatch! got %u, expect %u\n",hdr->seq, sip->rxseq-1);
		}

		if (SIP_HDR_IS_CTRL(hdr)) 
              {
			ret = sip_parse_events(sip, bufptr);

			//skb_pull(skb, hdr->len);
                     A_NETBUF_PULL(skb, hdr->len);
		}         
              else if (SIP_HDR_IS_DATA(hdr)) 
              {
			//struct esp_mac_rx_ctrl * mac_ctrl = NULL;
			//int pkt_len_enc = 0, buf_len = 0, pulled_len = 0;
			
//printf("seq %u \n", hdr->seq);

                     rskb = sip_parse_data_rx_info(sip, skb);
            
			if(rskb == NULL)
				goto _move_on;

//Need know upper layer recv is on.          
			if (sip->epub->wl.off == 0) 
                      {
                            
                            EglDataRx(sip->epub->p_netif, rskb);
			 } 
                      else 
                      {
				/* still need go thro parsing as skb_pull should invoke */
				A_NETBUF_FREE(rskb);
			 }
		} 

_move_on:
		if (hdr->len < remains_len) 
              {
			remains_len -= hdr->len;
		}
              else 
              {
			break;
		}
		bufptr += hdr->len;
	}

	do 
       {
//printf("skb free\n");
		A_NETBUF_FREE(skb);
	} while (0);

	return trigger_rxq;
}

static void _sip_rxq_process(struct esp_sip *sip)
{
        skb_t *skb = NULL;
        //u8* data;
        u8 sendup = false;

        egl_mutex_lock( &(sip->rx_lock) );
        while ((skb = A_NETBUF_DEQUEUE(&sip->rxq))) 
        {
                //printf("skb dequeue\n");
                if (sip_rx_pkt_process(sip, skb))
                        sendup = true;
        }
        egl_mutex_unlock( &(sip->rx_lock) );
      
                /* probably tx_credit is updated, try txq */
        sip_trigger_txq_process(sip);
}


        //  
        
void sip_rxq_process(struct esp_pub *epub)
{
       struct esp_sip *sip = epub->sip;

	if (sip->state == SIP_SEND_INIT) 
       {
		sip_send_chip_init(sip);
		sip->state = SIP_WAIT_BOOTUP;
              //printf("Send chip ini\n");
		return;
	}

        _sip_rxq_process(sip);

}



int sip_post_init(struct esp_sip *sip, struct sip_evt_bootup2 *bevt)
{
        struct esp_pub *epub;

        epub = sip->epub;

        sip->tx_aggr_buf = (u8 *)egl_malloc(SIP_TX_AGGR_BUF_SIZE);
        if (sip->tx_aggr_buf == NULL) 
        {
                printf("no mem for tx_aggr_buf! \n");
                return -1;
        }

//To target , the block sz is 260, actually, when send to target, the actual size is extended to 512.
//from  target, the block sz is 524, not used, the actual blokc size is 512. Fuck
printf(" tx_blksz: %d, rx_blksz: %d", bevt->tx_blksz, bevt->rx_blksz);

printf(" mac addr: ");
for(char i=0;i<6;i++)
printf("%x,", bevt->mac_addr[i]);

printf(" Opition: %x\n", bevt->options);
printf(" sip->credit_to_reserve: %x\n", bevt->credit_to_reserve);

        sip->tx_aggr_write_ptr = sip->tx_aggr_buf;

        sip->tx_blksz = bevt->tx_blksz;
        sip->rx_blksz = bevt->rx_blksz;
        sip->credit_to_reserve = bevt->credit_to_reserve;

//options=6,  0110
        sip->dump_rpbm_err = (bevt->options & SIP_DUMP_RPBM_ERR);
        sip->rxabort_fixed = (bevt->options & SIP_RXABORT_FIXED);
        sip->support_bgscan = (bevt->options & SIP_SUPPORT_BGSCAN);
        
        sip->sendup_rpbm_pkt = sip->dump_rpbm_err && false;

        /* print out MAC addr... */
        memcpy(epub->mac_addr, bevt->mac_addr, ETH_ALEN);
        
        //esp_sip_dbg(ESP_DBG_TRACE, "%s tx_blksz %d rx_blksz %d mac addr %pM\n", __func__, sip->tx_blksz, sip->rx_blksz, epub->mac_addr);

        return 0;
}





/***************ݵķ*********************/

void sip_free_ctrl_skbuff(struct esp_sip *sip, struct sk_buff *skb)
{
        memset(IEEE80211_SKB_CB(skb), 0, sizeof(struct ieee80211_tx_info));
        A_NETBUF_FREE(skb);
}

/* setup sip header and tx info, copy pkt into aggr buf */
static int sip_pack_pkt(struct esp_sip *sip, struct sk_buff *skb)
{
        struct ieee80211_tx_info *itx_info;
        struct sip_hdr *shdr;
        u32 tx_len = 0, offset = 0;
        bool is_data = true;

        itx_info = IEEE80211_SKB_CB(skb);

        if (itx_info->flags == 0xffffffff) 
        {
                shdr = (struct sip_hdr *)skb->data;
                is_data = false;
                tx_len = skb->len;
               // printf("offset %d skblen %d txlen %d\n",offset, skb->len, tx_len);
        } 
#if 1       
        else
        {
                //struct ieee80211_hdr *wh;
               // struct esp_vif *evif = (struct esp_vif *)itx_info->control.vif->drv_priv;
                shdr = (struct sip_hdr *)(sip->tx_aggr_write_ptr);
                /* update sip header */

                shdr->fc[0] = 0;
                shdr->fc[1] = 0;

               // if ((itx_info->flags & IEEE80211_TX_CTL_AMPDU) && esp_is_ip_pkt(skb))
                      //  SIP_HDR_SET_TYPE(shdr->fc[0], SIP_DATA_AMPDU);
                //else
                        SIP_HDR_SET_TYPE(shdr->fc[0], SIP_DATA);

#if 0
                /* make room for encrypted pkt */
                if (itx_info->control.hw_key)
                {

                        int alg = esp_cipher2alg(itx_info->control.hw_key->cipher);
                        if (unlikely(alg == -1)) 
                        {
                                //skb_queue_tail(&sip->epub->txdoneq, skb);
#ifndef FAST_TX_STATUS
                                /* TBD */
#else
                                sip_tx_status_report(sip, skb, itx_info, false);
                                atomic_dec(&sip->tx_data_pkt_queued);
                                return -1;
#endif /* FAST_TX_STATUS */
                        } 
                        else 
                        {
                                shdr->d_enc_flag = alg + 1;
                        }


                         shdr->d_hw_kid =  itx_info->control.hw_key->hw_key_idx | (evif->index<<7);
                } 
                else
#endif                    
                {
                        shdr->d_enc_flag=0;
                        shdr->d_hw_kid = 0;
                }
                 shdr->d_ac = 0;
                shdr->d_tid = 0;

                /* update sip tx info */
                //shdr->d_ac = skb_get_queue_mapping(skb);
                //shdr->d_tid = skb->priority & IEEE80211_QOS_CTL_TAG1D_MASK;
 //ҪȷҪҪd_ac,d_tidֵ

#if 0                
                wh = (struct ieee80211_hdr *)skb->data;
                if (ieee80211_is_mgmt(wh->frame_control)) 
                {
		/* addba/delba/bar may use different tid/ac */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3, 3, 0))
                        shdr->d_tid = 7;
#else  /* kernel > 3.3.0 */
                        if (ieee80211_is_beacon(wh->frame_control)) 
                        {
                                shdr->d_tid = 7;
                        }
#endif /* kernel < 3.3.0 */
                }
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 35))
                if (check_ac_tid(skb->data, shdr->d_ac, shdr->d_tid)) 
                {
                        shdr->d_ac = WME_AC_BE;
                        shdr->d_tid = 0;
                }
#endif  /* NEW_KERNEL || KERNEL_35 */
#endif

                /* make sure data is start at 4 bytes aligned addr. */
                offset = roundup(sizeof(struct sip_hdr), 4);
/*
#ifdef HOST_RC
                esp_sip_dbg(ESP_DBG_TRACE, "%s offset0 %d \n", __func__, offset);
                memcpy(sip->tx_aggr_write_ptr + offset, (void *)&itx_info->control, sizeof(struct sip_tx_rc));

                offset += roundup(sizeof(struct sip_tx_rc), 4);
                esp_show_tx_rates(&itx_info->control.rates[0]);

#endif 
*/
/*
                if (SIP_HDR_IS_AMPDU(shdr)) 
                {
                        memset(sip->tx_aggr_write_ptr + offset, 0, sizeof(struct esp_tx_ampdu_entry));
                        offset += roundup(sizeof(struct esp_tx_ampdu_entry), 4);
                }
*/

                tx_len = offset + skb->len;
                shdr->len = tx_len;  /* actual len */

                //esp_sip_dbg(ESP_DBG_TRACE, "%s offset %d skblen %d txlen %d\n", __func__, offset, skb->len, tx_len);
                //printf("offset %d skblen %d txlen %d\n",offset, skb->len, tx_len);

        }
#endif

//if(!(sip->txseq%1000))
   //     printf("3 tx_seq:%d\n",(sip->txseq/1000));

        shdr->seq = sip->txseq++;

        //esp_sip_dbg(ESP_DBG_TRACE, "%s seq %u, \n", __func__, shdr->seq);

        /* copy skb to aggr buf */
        memcpy(sip->tx_aggr_write_ptr + offset, skb->data, skb->len);

#if 0       //Add here for seq no checking.
unsigned int seq = 0;
unsigned short cur_len ;
{
       seq= *(unsigned int *)(sip->tx_aggr_write_ptr+8);
       cur_len = *(unsigned short *)(sip->tx_aggr_write_ptr+2);
       printf("%d->%d,%d,%d\n", seq,cur_len,skb->len,tx_len);

       if(seq == 5)
                printf("shdr add:%x, tx_ptr add:%x\n",shdr,sip->tx_aggr_write_ptr);
       if(skb->len == tx_len && seq>300)
       {
                printf("shdr add:%x, tx_ptr add:%x\n",shdr,sip->tx_aggr_write_ptr);
                printf("%d->%d,%d,%d\n", seq,cur_len,skb->len,tx_len);
                printf("show:");
                for(int k=0;k<20;k++)
                    printf("%x,",*(sip->tx_aggr_write_ptr+k));
                printf("\n");
       }
}

//printf("seq:%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",seq[0],seq[1],seq[2],seq[3],seq[4],
                 //seq[5],seq[6],seq[7],seq[8],seq[9],seq[10],seq[11],seq[12]);
#endif



        if (is_data) 
        {
#if 0    
                //skb_queue_tail(&sip->epub->txdoneq, skb);
#ifndef FAST_TX_STATUS
                /* store seq in driver data, need seq to pick pkt during tx status report */
                *(u32 *)itx_info->driver_data = shdr->seq;
                atomic_inc(&sip->pending_tx_status);
#else
                /* fake a tx_status and report to mac80211 stack to speed up tx, may affect
                 *  1) rate control (now it's all in target, so should be OK)
                 *  2) ps mode, mac80211 want to check ACK of ps/nulldata to see if AP is awake
                 *  3) BAR, mac80211 do BAR by checking ACK
                 */
                /*
                 *  XXX: need to adjust for 11n, e.g. report tx_status according to BA received in target
                 *
                 */
                sip_tx_status_report(sip, skb, itx_info, true);
                atomic_dec(&sip->tx_data_pkt_queued);

#endif /* FAST_TX_STATUS */
                STRACE_TX_DATA_INC();
#endif
//Know why no free for data ,because the free cause is done by upper layer. 
//But the cmd is alloc by the driver itself
                memset(IEEE80211_SKB_CB(skb), 0, sizeof(struct ieee80211_tx_info));
                A_NETBUF_FREE(skb);
        } 
        else 
        {
                /* no need to hold ctrl skb */
                sip_free_ctrl_skbuff(sip, skb);
                //STRACE_TX_CMD_INC();
        }

        /* TBD: roundup here or whole aggr-buf */
        tx_len = roundup(tx_len, sip->tx_blksz);

        sip->tx_aggr_write_ptr += tx_len;
        sip->tx_tot_len += tx_len;

        return 0;
}

void sip_trigger_txq_process(struct esp_sip *sip)
{
        if ( sip->tx_credits <= sip->credit_to_reserve )  //no credits, do nothing
                return;

        //if ( sip->epub->txq_stopped && sip->tx_credits >= 16 &&
           // !test_bit(ESP_WL_FLAG_STOP_TXQ, &sip->epub->wl.flags)) 
        if ( sip->epub->txq_stopped && sip->tx_credits >= 16 ) 
        {
                /* wakeup upper queue only if we have sufficient credits */                
                //esp_sip_dbg(ESP_DBG_TRACE, "%s wakeup ieee80211 txq \n", __func__);
                //atomic_set(&sip->epub->txq_stopped, false);
                sip->epub->txq_stopped = false;
                
                //ieee80211_wake_queues(sip->epub->hw);
// Annouce upper layer to send data now.
                UpperLayerTxSwitch(1);
                
        } 
        else if (sip->epub->txq_stopped ) 
        {
                //esp_sip_dbg(ESP_DBG_TRACE, "%s can't wake txq, credits: %d \n", __func__, atomic_read(&sip->tx_credits) );
               // printf("can't wake txq, credits:%d \n", sip->tx_credits);
        }

        //if (!skb_queue_empty(&sip->epub->txq)) 
        if (!A_NETBUF_QUEUE_EMPTY(&sip->epub->txq))
        {
//printf("tx queue work triggered\n");
                /* try to send out pkt already in sip queue once we have credits */
                //esp_sip_dbg(ESP_DBG_TRACE, "%s resume sip txq \n", __func__);
                //queue_work(sip->epub->esp_wkq, &sip->epub->tx_work);
                
                TXQueueWorkFlag++;
                //egl_sem_signal(&QueueWorkCntSem);
        }
}



/* write pkts in aggr buf to target memory */
static void sip_write_pkts(struct esp_sip *sip)
{
        int tx_aggr_len = 0;
        struct sip_hdr *first_shdr = NULL;
	 int err = 0;

     /*
        static int check = 0;
        check++;
        printf("t:%d\n",check);
    */

        tx_aggr_len = sip->tx_aggr_write_ptr - sip->tx_aggr_buf;
        
        if (tx_aggr_len < sizeof(struct sip_hdr)) 
        {
                printf(" tx_aggr_len %d \n", tx_aggr_len);
                //printk("%s tx_aggr_len %d \n", __func__, tx_aggr_len);
                //ASSERT(0);
        }
        //ASSERT(tx_aggr_len >= sizeof(struct sip_hdr));
        //ASSERT((tx_aggr_len & 0x3) == 0);

//Ȼǰѵһhdr ֵд
        first_shdr = (struct sip_hdr *)sip->tx_aggr_buf;

        //if (atomic_read(&sip->tx_credits) <= SIP_CREDITS_LOW_THRESHOLD) 
        if (sip->tx_credits <= SIP_CREDITS_LOW_THRESHOLD) 
        {
                first_shdr->fc[1] |= SIP_HDR_F_NEED_CRDT_RPT;
        }

        //ASSERT(tx_aggr_len == sip->tx_tot_len);
        if(tx_aggr_len != sip->tx_tot_len)
                printf(" tx_len err, tx_aggr_len:%d, sip->tx_tot_len:%d\n", tx_aggr_len,sip->tx_tot_len);
        
        /* still use lock bus instead of sif_lldesc_write_sync since we want to protect several global varibles assignments */
        //sif_lock_bus(sip->epub);
//show_buf(sip->tx_aggr_buf,tx_aggr_len);

#if 0       //Add here for seq no checking.
unsigned int seq[31] = {0};
unsigned short cur_len ;

char t;
unsigned int temp_len = 0;
printf("t_len:%d, seq:",tx_aggr_len);
for(t=0;t<31;t++)
{
       seq[t]= *(unsigned int *)(sip->tx_aggr_buf+temp_len+8);
       cur_len = *(unsigned short *)(sip->tx_aggr_buf+temp_len+2);
       //temp_len += *(unsigned short *)(tx_data+temp_len+2);
       temp_len += (cur_len%260)!=0 ? (cur_len/260+1)*260:cur_len ;
       printf("%d->%d,",seq[t],cur_len);
       if(temp_len >= tx_aggr_len)
            break;
}
printf("\n");

//printf("seq:%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",seq[0],seq[1],seq[2],seq[3],seq[4],
                 //seq[5],seq[6],seq[7],seq[8],seq[9],seq[10],seq[11],seq[12]);
#endif

        err =  sif_lldesc_write_sync(sip->epub, sip->tx_aggr_buf, tx_aggr_len);
        
        //err = sif_lldesc_write_raw(sip->epub, sip->tx_aggr_buf, tx_aggr_len);
        
        sip->tx_aggr_write_ptr = sip->tx_aggr_buf;
        sip->tx_tot_len = 0;

        //sif_unlock_bus(sip->epub);

	if (err)
        ;
		//esp_sip_dbg(ESP_DBG_ERROR, "func %s err: %d\n", __func__, err);
}

bool sip_tx_data_need_stop(struct esp_sip *sip)
{
        return (sip->pending_tx_status >= SIP_PENDING_STOP_TX_THRESHOLD);
}


static void sip_after_write_pkts(struct esp_sip *sip)
{
        if( sip->data_tx_stopped == 0 && sip_tx_data_need_stop(sip) ) 
        {
                printf("data_tx_stopped \n");
                sip->data_tx_stopped =  true;
        }
}


    //  ݵķ
    
void sip_txq_process(struct esp_pub *epub)
{
        struct sk_buff *skb;
        struct esp_sip *sip = epub->sip;
        u32 pkt_len = 0, tx_len = 0, blknum = 0;
        bool queued_back = false;
        bool out_of_credits = false;
        struct ieee80211_tx_info *itx_info;
        
//printf("TX queue triggered\n");
//printf("2,");
#if 1
        while (skb = A_NETBUF_DEQUEUE(&epub->txq))
        {
                /* cmd skb->len does not include sip_hdr too */
                pkt_len = skb->len;                
                itx_info = IEEE80211_SKB_CB(skb);
/*
                if (itx_info->flags != 0xffffffff) 
                {
                        pkt_len += roundup(sizeof(struct sip_hdr), 4);
                        if (itx_info->flags & IEEE80211_TX_CTL_AMPDU)
                                pkt_len += roundup(sizeof(struct esp_tx_ampdu_entry), 4);
                }
*/
                if (itx_info->flags != 0xffffffff) 
                        pkt_len += roundup(sizeof(struct sip_hdr), 4);
                /* current design simply requires every sip_hdr must be at the begin of mblk, that definitely
                 * need to be optimized, e.g. calulate remain length in the previous mblk, if it larger than
                 * certain threshold (e.g, whole pkt or > 50% of pkt or 2 x sizeof(struct sip_hdr), append pkt
                 * to the previous mblk.  This might be done in sip_pack_pkt()
                 */
                 
                pkt_len = roundup(pkt_len, sip->tx_blksz);
                blknum = pkt_len / sip->tx_blksz;                       //one blk, blksz bytes.
                
                //esp_dbg(ESP_DBG_TRACE, "%s skb_len %d pkt_len %d blknum %d\n", __func__, skb->len, pkt_len, blknum);
//printf("3: %d,%d,",blknum,sip->tx_credits - sip->credit_to_reserve);
                if ( blknum > (sip->tx_credits - sip->credit_to_reserve) ) 
                {
                        //esp_dbg(ESP_DBG_TRACE, "%s out of credits!\n", __func__);
                        //STRACE_TX_OUT_OF_CREDIT_INC();
                        //printf("Tx out of credits \n");
                        
                        queued_back = true;
                        out_of_credits = true;

 //Announce upper layer to stop send data down,
 
                       // if (epub->hw) 
                        {
                               // ieee80211_stop_queues(epub->hw);
                               UpperLayerTxSwitch(0);
//Stop enqueue  need handle             

                               //atomic_set(&epub->txq_stopped, true);
                                epub->txq_stopped = true;
                        }
                        /* we will be back */
                        break;
                }
//printf("4:%d,",tx_len);
                if (tx_len >= 32 * 1024) 
                //if (tx_len >= (1<<15)) 
                {
                        //esp_dbg(ESP_DBG_TRACE, "%s too much pkts in one shot!\n", __func__);
                        //STRACE_TX_ONE_SHOT_INC();
                        printf("too much pkts in one shot!\n");
                        queued_back = true;
                        /* we will be back */
                        break;
                }
//printf("5,");
                tx_len += pkt_len;
                if (tx_len >= SIP_TX_AGGR_BUF_SIZE) 
                {
//printf("6,");
                        /* do we need to have limitation likemax 8 pkts in a row? */
                        tx_len -= pkt_len;
                        queued_back = true;
                        //printf("TX_BUF_FULL!\n");
                        break;
                }


                if (sip_pack_pkt(sip, skb) != 0) 
                {
                        /* wrong pkt, won't send to target */
                        tx_len -= pkt_len;
                        continue;
                }

                //esp_sip_dbg(ESP_DBG_TRACE, "%s:before sub, credits is %d\n", __func__, atomic_read(&sip->tx_credits));
                //atomic_sub(blknum, &sip->tx_credits);
                sip->tx_credits -= blknum;
                //esp_sip_dbg(ESP_DBG_TRACE, "%s:after sub %d,credits remains %d\n", __func__, blknum, atomic_read(&sip->tx_credits));
        }

//tx_len>16k  and  blknum > reset credits.
        if (queued_back) 
        {
                //skb_queue_head(&epub->txq, skb);
                A_NETBUF_ENQUEUE_HEAD(&epub->txq, skb);
        }
        
//printf("\n",tx_len);

//Send data out here 
        if (tx_len)
        {
                sip_write_pkts(sip);
                sip_after_write_pkts(sip);
        }

//tx_len > 16K
        if (queued_back && !out_of_credits) 
        {
                sip_trigger_txq_process(sip);
        }
        
#endif

}



     //  ݷ: 

struct sk_buff * sip_alloc_ctrl_skbuf(struct esp_sip *sip, u16 len, u32 cid) 
{
        struct sip_hdr *si = NULL;
        struct ieee80211_tx_info *ti = NULL;
        struct sk_buff *skb = NULL;

        //ASSERT(len <= sip->tx_blksz);
        /* no need to reserve space for net stack */

        skb = A_NETBUF_ALLOC(len);

        if (skb == NULL) 
        {
                //esp_dbg(ESP_DBG_ERROR, "no skb for ctrl !\n");
                printf("No skb for ctrl\n");
                return NULL;
        }

        skb->len = len;

        ti = IEEE80211_SKB_CB(skb);
        /* set tx_info flags to 0xffffffff to indicate sip_ctrl pkt */
        ti->flags = 0xffffffff;
       
        si = (struct sip_hdr *)skb->data;
        memset(si, 0, sizeof(struct sip_hdr));
        SIP_HDR_SET_TYPE(si->fc[0], SIP_CTRL);
        si->len = len;
        si->c_cmdid = cid;

        return skb;
}


int sip_cmd_enqueue(struct esp_sip *sip, struct sk_buff *skb)
{

	if (!sip || !sip->epub)
      {
		//esp_dbg(ESP_DBG_ERROR, "func %s, sip->epub->txq is NULL\n", __func__);
		return -1;
	}

	if (!skb)
       {
		//esp_dbg(ESP_DBG_ERROR, "func %s, skb is NULL\n", __func__);
		return -2;
	}

        A_NETBUF_ENQUEUE_TAIL(&sip->epub->txq, skb);

        TXQueueWorkFlag++;
        //printf("TXQ Flag:%d\n",TXQueueWorkFlag);
        //egl_sem_signal(&QueueWorkCntSem);

        return 0;
}

int sip_tx_data_pkt_enqueue(struct esp_pub *epub, struct sk_buff *skb)
{
	if(!epub || !epub->sip) 
       {
		if (!epub)
			//esp_dbg(ESP_DBG_ERROR, "func %s, epub is NULL\n", __func__);
        		printf("func: tx_data_pkt_enqueue, epub is NULL\n");
		else
			//esp_dbg(ESP_DBG_ERROR, "func %s, epub->sip is NULL\n", __func__);
			printf("func: tx_data_pkt_enqueue,  epub->sip is NULL\n");
		return -1;
	}
	if (!skb) {
                //esp_dbg(ESP_DBG_ERROR, "func %s, skb is NULL\n", __func__);
                printf("func: tx_data_pkt_enqueue,  skb is NULL\n");
                return -2;
        }
        //skb_queue_tail(&epub->txq, skb);
        A_NETBUF_ENQUEUE_TAIL(&epub->txq, skb);
        //atomic_inc(&epub->sip->tx_data_pkt_queued);
        epub->sip->tx_data_pkt_queued++;
        return 0;
}



