/*
 *  Copyright (c) 2009- 2012 Espressif System.
 *
 *    Serial Interconnctor Protocol
 */

#ifndef __ESP_SIP_H__
#define __ESP_SIP_H__

#include "sys/egl_sys_arch.h"
#include "sip2_common.h"
#include "linux_inc\list.h"

#include "egl_netbuf.h"

#define SIP_PKT_MAX_LEN (1024*16)

/* 16KB on normal X86 system, should check before porting to orhters */
//#define PAGE_SIZE (1<<12)      // 4kb
/*For ARM  page size is 0.5k*/
#define PAGE_SIZE (4096)      // 512byte

#define SIP_TX_AGGR_BUF_SIZE (4 * PAGE_SIZE)
#define SIP_RX_AGGR_BUF_SIZE (4 * PAGE_SIZE)

struct sk_buff;

typedef enum SIP_STATE {
        SIP_INIT = 0,
        SIP_PREPARE_BOOT,
        SIP_BOOT,
        SIP_SEND_INIT,
        SIP_WAIT_BOOTUP,
        SIP_RUN,
        SIP_SUSPEND,
        SIP_STOP
} SIP_STATE;

enum sip_notifier {
        SIP_TX_DONE = 1,
        SIP_RX_DONE = 2,
};

#define SIP_CREDITS_LOW_THRESHOLD  64  //i.e. 4k


struct esp_sip {
        struct list_head free_ctrl_txbuf;
        struct list_head free_ctrl_rxbuf;

        u32 rxseq; /* sip pkt seq, should match target side */
        u32 txseq;

        SIP_STATE state;
        //spinlock_t lock;
        egl_mutex_t lock;
        
        int boot_credits;
        u8 tx_credits;                      //How many block need to be sent down
        u8 credit_to_reserve;           //a const, must reserve 3 blocks ,not to sent
        
       // atomic_t tx_ask_credit_update;

        u8 * rawbuf;  /* used in boot stage, free once chip is fully up */
        u8 * tx_aggr_buf;
        u8 * tx_aggr_write_ptr;  /* update after insertion of each pkt */
        u8 * tx_aggr_lastpkt_ptr;

        u8 * rx_aggr_buf;
        u8 * rx_aggr_write_ptr;
        u8 * rx_aggr_lastpkt_ptr;

        egl_mutex_t rx_lock;
        A_NETBUF_QUEUE_T rxq;
       
        u16 tx_blksz;
        u16 rx_blksz;

        bool dump_rpbm_err;
        bool sendup_rpbm_pkt;
        bool rxabort_fixed;
        bool support_bgscan;
        
        u32 tx_tot_len; /* total len for one transaction */
        u32 rx_tot_len;

       u8 rx_handling;
       u8 tx_data_pkt_queued;
       
       u8 pending_tx_status;
       u8 data_tx_stopped;
       u8 tx_stopped;
       struct esp_pub *epub;
};


struct sip_pkt {
        struct list_head list;

        u8 * buf_begin;
        u32  buf_len;
        u8 * buf;
        u32  payload_len;
#if 0
        union {
                struct sip_tx_pkt_info tx;
                struct sip_rx_pkt_info rx;
        } info;
#endif
        void *context;
        int status;
        //void (* completion)(struct esp_sip *sip, struct sip_pkt *pkt);
};


int sip_rx(struct esp_pub * epub);
void sip_rxq_process(struct esp_pub *epub);
int sip_post_init(struct esp_sip *sip, struct sip_evt_bootup2 *bevt);
void sip_txq_process(struct esp_pub *epub);

int sip_cmd_enqueue(struct esp_sip *sip, struct sk_buff *skb);
int sip_tx_data_pkt_enqueue(struct esp_pub *epub, struct sk_buff *skb);
    
struct sk_buff * sip_alloc_ctrl_skbuf(struct esp_sip *sip, u16 len, u32 cid) ;
//int sip_download_fw(struct esp_sip *sip, u32 load_addr, u32 boot_addr);


#endif
