#include <stdio.h>
#include "sip2_common.h"

#include "sys/egl_sys_arch.h"
#include "port/port_arch.h"



void egl_init(void)
{

}

int egl_sem_new(egl_sem_t *sem, u8 count)
{
	int xReturn = 1;
	vSemaphoreCreateBinary( *sem );
    
        if( (*sem) != NULL )
        {
                if(count == 0)	// Means it can't be taken
                {
                       xSemaphoreTake(*sem,1);
                }
                xReturn = 0;
        }
        else
        {
               ;	// TBD need assert
        }
	return xReturn;
}

int egl_cntsem_new(egl_sem_t *sem, u8 count)
{
	int xReturn = 1;
	*sem = xSemaphoreCreateCounting( 50,0 );
    
        if( (*sem) != NULL )
        {
                if(count == 0)	// Means it can't be taken
                {
                       xSemaphoreTake(*sem,1);
                }
                xReturn = 0;
        }
        else
        {
               ;	// TBD need assert
        }
	return xReturn;
}

/*-----------------------------------------------------------------------------------*/
// Deallocates a semaphore
void egl_sem_free(egl_sem_t *sem)
{
	//vQueueDelete( sem );
        vSemaphoreDelete(*sem);
}


/*-----------------------------------------------------------------------------------*/
// Signals a semaphore
void egl_sem_signal(egl_sem_t *sem)
{
	xSemaphoreGive( *sem );
}


/*-----------------------------------------------------------------------------------*/
/*
  Blocks the thread while waiting for the semaphore to be
  signaled. If the "timeout" argument is non-zero, the thread should
  only be blocked for the specified time (measured in
  milliseconds).

  If the timeout argument is non-zero, the return value is the number of
  milliseconds spent waiting for the semaphore to be signaled. If the
  semaphore wasn't signaled within the specified time, the return value is
  SYS_ARCH_TIMEOUT. If the thread didn't have to wait for the semaphore
  (i.e., it was already signaled), the function may return zero.

  Notice that lwIP implements a function with a similar name,
  sys_sem_wait(), that uses the sys_arch_sem_wait() function.
*/
u32 egl_arch_sem_wait(egl_sem_t *sem, u32 timeout)
{
        portTickType StartTime, EndTime, Elapsed;
        unsigned long ulReturn;
        
	StartTime = xTaskGetTickCount();
	
	if(	timeout != 0)
	{
		if( xSemaphoreTake( *sem, timeout/ portTICK_RATE_MS ) == pdTRUE )
		{
			EndTime = xTaskGetTickCount();
			Elapsed = (EndTime - StartTime)* portTICK_RATE_MS;
			if( Elapsed == 0 )
			{
				Elapsed = 1;
			}
                     ulReturn = Elapsed;
		}
		else
		{
			ulReturn = SYS_ARCH_TIMEOUT;
		}
	}
	else // must block without a timeout
	{
		//while( xSemaphoreTake( *sem, portMAX_DELAY ) != pdTRUE );
        	xSemaphoreTake( *sem, portMAX_DELAY );
            
		EndTime = xTaskGetTickCount();
		Elapsed = ( EndTime - StartTime ) * portTICK_RATE_MS;
		if( Elapsed == 0 )
		{
			Elapsed = 1;
		}
              ulReturn = Elapsed;
	 }
    
        return ulReturn ; // return time blocked	
}

/*-----------------------------------------------------------------------------------*/
//  Creates an empty mailbox.
int egl_mbox_new(egl_mbox_t *mbox, int size)
{
int xReturn = 1;
        
        *mbox = xQueueCreate( size, sizeof( void * ));

        if( *mbox!=NULL )
        {
                xReturn = 0;
        }
        
        return xReturn;
}

/*-----------------------------------------------------------------------------------*/
/*
  Deallocates a mailbox. If there are messages still present in the
  mailbox when the mailbox is deallocated, it is an indication of a
  programming error in lwIP and the developer should be notified.
*/
void egl_mbox_free(egl_mbox_t *mbox)
{
	if( uxQueueMessagesWaiting( *mbox ) )
	{
		/* Line for breakpoint.  Should never break here! */
//		__asm volatile ( "NOP" );
	}

	vQueueDelete( *mbox );
}


/*-----------------------------------------------------------------------------------*/
//   Posts the "msg" to the mailbox.
void egl_mbox_post(egl_mbox_t *mbox, void *msg)
{
	while( xQueueSendToBack( *mbox, &msg, portMAX_DELAY ) != pdTRUE );
}

int egl_mbox_trypost(egl_mbox_t *mbox, void *msg)
{
int xReturn;

        if( xQueueSend( *mbox, &msg, ( portTickType )0) == pdPASS)  
        {
                xReturn = 0;
        }
        else
        {
                xReturn = 1;
        }
        return xReturn;
}



/*-----------------------------------------------------------------------------------*/
/*
  Blocks the thread until a message arrives in the mailbox, but does
  not block the thread longer than "timeout" milliseconds (similar to
  the sys_arch_sem_wait() function). The "msg" argument is a result
  parameter that is set by the function (i.e., by doing "*msg =
  ptr"). The "msg" parameter maybe NULL to indicate that the message
  should be dropped.

  The return values are the same as for the sys_arch_sem_wait() function:
  Number of milliseconds spent waiting or SYS_ARCH_TIMEOUT if there was a
  timeout.

  Note that a function with a similar name, sys_mbox_fetch(), is
  implemented by lwIP.
*/
u32 egl_arch_mbox_fetch(egl_mbox_t *mbox, void **msg, u32 timeout)
{
void *dummyptr;
portTickType StartTime, EndTime, Elapsed;
unsigned long ulReturn;

	StartTime = xTaskGetTickCount();

	if( msg == NULL )
	{
		msg = &dummyptr;
	}
		
	if(	timeout != 0 )
	{
		if(pdTRUE == xQueueReceive( *mbox, &(*msg), timeout / portTICK_RATE_MS )  )
		{
			EndTime = xTaskGetTickCount();
			Elapsed = (EndTime - StartTime)* portTICK_RATE_MS;
			if( Elapsed == 0 )
			{
				Elapsed = 1;
			}
            
                     ulReturn = Elapsed;
		}
		else // timed out blocking for message
		{
			*msg = NULL;
			ulReturn = SYS_ARCH_TIMEOUT;
		}
	}
	else // block forever for a message.
	{
		while( pdTRUE != xQueueReceive( *mbox, &(*msg), portMAX_DELAY ) );

		EndTime = xTaskGetTickCount();
		Elapsed = (EndTime - StartTime)* portTICK_RATE_MS;
		if( Elapsed == 0 )
		{
			Elapsed = 1;
		}
              ulReturn = Elapsed;
	}
    
       return ulReturn ; // return time blocked TBD test	
}


u32 egl_arch_mbox_tryfetch(egl_mbox_t *mbox, void **msg)
{
void *pvDummy;
unsigned long ulReturn;

        if( msg== NULL )
        {
                msg = &pvDummy;
        }
        
        if( pdTRUE == xQueueReceive( *mbox, &(*msg), 0 ) ) 
	{
		ulReturn = 0;
	}
	else
	{
		ulReturn = SYS_MBOX_EMPTY;
	}

	return ulReturn;
}



/** Create a new mutex
 * @param mutex pointer to the mutex to create
 * @return a new mutex */
int egl_mutex_new(egl_mutex_t *mutex)
{
int xReturn = 1;

	*mutex = xSemaphoreCreateMutex();

	if( *mutex != NULL ) 
	{
		xReturn = 0;
		
	} 
	else 
	{
		;
	}
	
	return xReturn;
}

/** Lock a mutex
 * @param mutex the mutex to lock */
void egl_mutex_lock(egl_mutex_t *mutex)
{
	while( xSemaphoreTake( *mutex, portMAX_DELAY ) != pdPASS );
}

/** Unlock a mutex
 * @param mutex the mutex to unlock */
void egl_mutex_unlock(egl_mutex_t *mutex)
{
	xSemaphoreGive( *mutex );
}


/** Delete a semaphore
 * @param mutex the mutex to delete */
void egl_mutex_free(egl_mutex_t *mutex)
{
	vQueueDelete( *mutex );
}

u32 egl_now(void)
{
	return xTaskGetTickCount();
}

void egl_delay(portTickType xTicksToDelay )
{
	vTaskDelay( xTicksToDelay );
}

void egl_thread_del(egl_thread_t thread)
{
        vTaskDelete(thread);
}

/*-----------------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------------*/
// TBD
/*-----------------------------------------------------------------------------------*/
/*
  Starts a new thread with priority "prio" that will begin its execution in the
  function "thread()". The "arg" argument will be passed as an argument to the
  thread() function. The id of the new thread is returned. Both the id and
  the priority are system dependent.
*/
egl_thread_t egl_thread_new(const char *name, egl_thread_fn thread, void *arg, int stacksize, int prio)
{
        xTaskHandle CreatedTask;
        portBASE_TYPE result;

	result = xTaskCreate(thread, ( signed char * )name, stacksize, arg, prio, &CreatedTask );

	if(result == pdPASS)
	{
		return CreatedTask;
	}
	else
	{
		return NULL;
	}
}

/*
void egl_thread_enter_critical(void)  
{
        taskENTER_CRITICAL();
}
void egl_thread_exit_critical(void)  
{
        taskEXIT_CRITICAL();
}
*/

void egl_thread_set_prio(egl_thread_t task, int prio)
{
        vTaskPrioritySet( task, prio );
}

int egl_thread_get_prio(egl_thread_t task) 
{
        return uxTaskPriorityGet(task);   
}

void egl_thread_yield(void)
{
        taskYIELD();
}

/*
*This timer new func is autoreload type .
*
*/
/*
egl_timer_t egl_timer_new(const signed char *name, egl_timer_fn timer, unsigned long ticks)
{
        return xTimerCreate( name, ( ticks / portTICK_RATE_MS), pdTRUE, 0,timer);
}

int egl_timer_start(egl_timer_t timer)
{
portBASE_TYPE result;
         result = xTimerStart( timer, 100 );
         if( result==pdPASS )
                return 0;
         else
                return 1; 
}

int egl_timer_stop(egl_timer_t timer)
{
portBASE_TYPE result;
        result = xTimerStop( timer, 100 );
         if( result==pdPASS )
                return 0;
         else
                return 1; 
}

int egl_timer_del(egl_timer_t timer)
{
portBASE_TYPE result;
        result = xTimerDelete( timer, 100 );
         if( result==pdPASS )
                return 0;
         else
                return 1; 
}
*/

void *egl_malloc(u32 size )
{
      return pvPortMalloc(size);
       //return pvPortMalloc_1( size );
       //         return (void *)malloc( size );
}

void egl_free(void *bufPtr )
{
        vPortFree( bufPtr );
        //vPortFree_1( bufPtr );
        //        free( bufPtr );
}



