#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sys/egl_sys_arch.h"
#include "bus/egl_bus_hw.h"

#include "sdio_cfg.h"

#if SDIO_SUPPORT

egl_mutex_t BusHostMutex;
egl_sem_t BusIrqReadSem;

extern void *Memcpy(void *dest, const void *src, size_t count);

/*
1 SPI/NVIC_GPIO,NVIC,DMA,SPI hardware ini.
2 SPI bus cmd ini.
3 BusHoldMutex ini.
4 BusIrqReadSem ini.
*/
void bus_ini(void)
{
        SdioHwIni();
        egl_delay(5);
        SdioProtocolIni();

        if( egl_mutex_new(&BusHostMutex)==0)
        {
                printf("BusHostMutex ini,ok\n");
        }
        if( egl_sem_new(&BusIrqReadSem,0)==0)
        {
                printf("BusIrqReadSem ini,ok\n");
        }
}

void bus_claim_host(void )
{
        egl_mutex_lock(&BusHostMutex);
}


void bus_release_host(void)
{
        egl_mutex_unlock(&BusHostMutex);
}


void bus_irq_en(u8 enable)
{
        //SdioIrqEnable( enable );
}

 u8 BusReadReg(u32 addr, int *err_ret )
{
        return CMD52Read(1, addr);
}


int BusWriteReg(u32 addr, u8 value)
{
        CMD52Write(1, addr, value);
        return 0;
}



/**
 *	sdio_writesb - write to a FIFO of a SDIO function
 *	@func: SDIO function to access
 *	@addr: address of (single byte) 
 *	@src: buffer that contains the data to write
 *	@count: number of bytes to write
 *
 *	Writes to the specified FIFO of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */
 
int BusWriteBytes(u32 addr, void *src,u32 count)
{
       return 0;
}

/**
 *	sdio_memcpy_toio - write a chunk of memory to a SDIO function
 *	@func: SDIO function to access
 *	@addr: address to start writing to
 *	@src: buffer that contains the data to write
 *	@count: number of blocks to write
 *
 *	Writes to the address space of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */
int BusWriteBlocks(u32 addr,void *src, u32 count)
{
        SD_Error sd_ret =  CMD53WriteBlocks(addr, (u32 *)src, count);

        //printf("w_ret:%d\n",sd_ret);
        if(sd_ret == SD_OK) 
                return 0;
        else
                return -1;
}


/**
 *	sdio_readsb - read from a FIFO on a SDIO function
 *	@func: SDIO function to access
 *	@dst: buffer to store the data
 *	@addr: address of (single byte) FIFO  addr:0~0xfffff
 *	@count: number of bytes to read        count<=512
 *
 *	Reads from the specified FIFO of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */  
int BusReadBytes(u32 addr, void *dst, u32 count)
{
    SD_Error sd_ret =  CMD53ReadBytes(addr, (u32 *)dst, count);
        if(sd_ret == SD_OK) 
                return 0;
        else
                return -1;
}


/**
 *	sdio_memcpy_fromio - read a chunk of memory from a SDIO function
 *	@func: SDIO function to access
 *	@dst: buffer to store the data
 *	@addr: address to begin reading from
 *	@count: number of bytes to read
 *
 *	Reads from the address space of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */
int BusReadBlocks(u32 addr, void *dst, u32 count)
{
       SD_Error sd_ret =  CMD53ReadBlocks(addr, (u32 *)dst, count);
        if(sd_ret == SD_OK) 
                return 0;
        else
                return -1;
}
#endif

