#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sys/egl_sys_arch.h"
#include "bus/egl_bus_hw.h"
#include "spi_cfg.h"
#ifdef SPI_SUPPORT
#define FREQ (9000000)    

#if (FREQ == 18000000)
#define CMD_RESP_SIZE (10)    //Common respon wait time 6
#define DATA_RESP_SIZE_W (207+62)    //Only for Write bytes function, data write response.  max:207+62
#define DATA_RESP_SIZE_R (204+61)    //Only for Read bytes function, data write response    max:204+61

#define BLOCK_W_DATA_RESP_SIZE_EACH          (20)           //For each data write resp size, in block write , 20
#define BLOCK_W_DATA_RESP_SIZE_FINAL     (208+62)   //For final data write resp size, in block write ,max: 208+62

#define BLOCK_R_DATA_RESP_SIZE_1ST          (204+61)    //For each data read resp size, in block read  ,max: 204+61
#define BLOCK_R_DATA_RESP_SIZE_EACH       (20)   //For each data read resp size, in block read , 20

#elif(FREQ == 9000000)      

#define CMD_RESP_SIZE (10)    //Common respon wait time
#define DATA_RESP_SIZE_W (103+40)    //Only for Write bytes function, data write response.  max: 103
#define DATA_RESP_SIZE_R (118+40)    //Only for Read bytes function, data write response  max: 118
//w: oxFF : 218 clock.     oxFE : 214 clock.  

#define BLOCK_W_DATA_RESP_SIZE_EACH          (20)           //For each data write resp size, in block write 
#define BLOCK_W_DATA_RESP_SIZE_FINAL     (112+40)    //For final data write resp size, in block write ,max :112

#define BLOCK_R_DATA_RESP_SIZE_1ST          (123+40)   //For each data read resp size, in block read ,max: 123
#define BLOCK_R_DATA_RESP_SIZE_EACH       (20)   //For each data read resp size, in block read 

#endif
#define MAX_BUF_SIZE        (5*1024)

u8 *tx_cmd;
u8 *rx_cmd; 
u8 comm_buff[5*1024];
u8 *copy_buff;
egl_mutex_t BusHostMutex;
egl_sem_t BusIrqReadSem;

//u16 SPI_BLOCK_SIZE=512;
//#define test
#ifdef test
int max_dataW_resp_size;
int max_dataR_resp_size;
int max_block_dataW_resp_size;
int max_block_dataR_resp_size;
#endif

extern void *Memcpy(void *dest, const void *src, size_t count);

/*
1 SPI/NVIC_GPIO,NVIC,DMA,SPI hardware ini.
2 SPI bus cmd ini.
3 BusHoldMutex ini.
4 BusIrqReadSem ini.
*/
void bus_ini(void)
{
        SpiHwIni();
        egl_delay(5);
        SpiProtocolIni();

        if( egl_mutex_new(&BusHostMutex)==0)
        {
                printf("BusHostMutex ini,ok\n");
        }
        if( egl_sem_new(&BusIrqReadSem,1)==0)
        {
                printf("BusIrqReadSem ini,ok\n");
        }
		
        copy_buff=(u8 *)egl_malloc(MAX_BUF_SIZE);//egl_malloc pvPortMalloc
         tx_cmd = (u8 *)egl_malloc(MAX_BUF_SIZE) ;//comm_buff
         rx_cmd= (u8 *)egl_malloc(MAX_BUF_SIZE);
          //rx_cmd = (u8 *)egl_malloc(MAX_BUF_SIZE);
         printf("temp addr: %x\n",rx_cmd);
}

void bus_claim_host(void )
{
        egl_mutex_lock(&BusHostMutex);
}


void bus_release_host(void)
{
        egl_mutex_unlock(&BusHostMutex);
}


void bus_irq_en(u8 enable)
{
        SpiIrqEnable( enable );
}

 u8 BusReadReg(u32 addr, int *err_ret )
{

        tx_cmd[0]=0x74;

        if( addr >= (1<<17) )
        {
                *err_ret = 1;
                return 0;
        }
        else
        {
                tx_cmd[1]=0x10|0x00|(addr>>15);    
                tx_cmd[2]=addr>>7;    

                tx_cmd[3]=( addr<<1 );
                tx_cmd[4]= 0x00;    
        }
        
        tx_cmd[5]=0x01;

        STSpiWrite(tx_cmd,6);
                //printf("CMD52 Read \n");


#if 0
//Response read
        memset(rx_cmd,0xFF,20);
        ST_Spi_Read(rx_cmd,20);  
        
        //printf("Data Read :");     
        for(i=0;i<20;i++)
        {
            printf("%X,",rx_cmd[i]);
        }
        printf("\n");
        return 0;
 #else

 //Read Response.        
        do
        {
                rx_cmd[0]=0xFF;
                STSpiRead(rx_cmd,1);
         }while(rx_cmd[0]==0xFF);
        rx_cmd[1]=0xFF;
                STSpiRead(rx_cmd+1,1);

        if(rx_cmd[0]!=0)
            *err_ret = 2;
        else
            *err_ret = 0;
        return rx_cmd[1];        
 #endif



}


int BusWriteReg(u32 addr, u8 value,u8 func)
{
        tx_cmd[0]=0x74;

        if( addr >= (1<<17) )
        {
                return 1;
        }
         if( func >= 7 )
        {
                return -1;
        }
        else
        {
                tx_cmd[1]=0x80|(func<<4)|0x00|(addr>>15);    //0x94;
                tx_cmd[2]=addr>>7;    

                tx_cmd[3]=( addr<<1 );
                tx_cmd[4]= value;
        }
        
        tx_cmd[5]=0x01;
        STSpiWrite(tx_cmd,6);
                //printf("CMD52 <Write config> stauts=16 \n");


#if 0
        memset(rx_cmd,0xFF,20);
                ST_Spi_Read(rx_cmd,20);

        for(i=0;i<15;i++)
        {
            printf("%X,",rx_cmd[i]);
        }
        printf("\n"); 
        return 0;
#else

        do
        {
                rx_cmd[0]=0xFF;
                STSpiRead(rx_cmd,1);
         }while(rx_cmd[0]==0xFF);
        rx_cmd[1]=0xFF;
                STSpiRead(rx_cmd+1,1);

         if( rx_cmd[1]!=value )    
         {
                printf("Write error,%X,%X\n", rx_cmd[0],rx_cmd[1]);
                return 2;
         }

        return 0;
        
#endif
//Read Response.        

}

int sif_spi_write_bytes(u32 addr, u8*src,u16 count,u8 func)
{
    int i;
    int pos,data_resp_size_w,total_num;       
    unsigned char *tx_data = src;
    int err_ret = 0;     
    data_resp_size_w =((((((count>>2)+1) *25)>>5)*21+16)>>3)+1+10;// ((((((count>>5)+1) *75)) +44)>>3) +1;

    if(count > 512)
    {
        err_ret = -1;
        return err_ret;
    }
    if(func > 7)
    {
        err_ret = -3;
        return err_ret;
    }
    tx_cmd[0]=0x75;

    if( addr >= (1<<17) )
    {
        err_ret = -2;
        return err_ret;
    }
    else
    {
        tx_cmd[1]=0x80|(func<<4)|0x04|(addr>>15);    //0x94;
        tx_cmd[2]=addr>>7;    

        if(count == 512 )
        {
            tx_cmd[3]=( addr<<1|0x0 );
            tx_cmd[4]= 0x00;     //0x08;
        }
        else
        {
            tx_cmd[3]=( addr<<1|(count>>8 & 0x01) );
            tx_cmd[4]= count & 0xff;     //0x08;
        }
    }

    tx_cmd[5]=0x01;
    pos = 5+1;

    //Add cmd respon
    memset(tx_cmd+pos,0xff,CMD_RESP_SIZE);
    pos =pos+ CMD_RESP_SIZE;

    //Add token        
    tx_cmd[pos]=0xFE;
    pos = pos+1;

    //Add data  
    memcpy(tx_cmd+pos,tx_data,count);
    pos = pos+count;

    //Add data respon
    memset(tx_cmd+pos,0xff,data_resp_size_w);
    total_num= pos+ data_resp_size_w ;
    memcpy(copy_buff,tx_cmd,pos);
    STSpiWrite(tx_cmd,total_num);

    pos = 5+1;
    for(i=0;i<CMD_RESP_SIZE;i++)
    {
        if(tx_cmd[pos+i] == 0x00 && tx_cmd[pos+i-1] == 0xff)
        {
            if(tx_cmd[pos+i+1] == 0x00 && tx_cmd[pos+i+2] == 0xff)
                break;      
        }
    }

    if(i>=CMD_RESP_SIZE)
    {
		printf("sif_spi_write_bytes send:\n");
        for(i=0;i<total_num;i++)
          {
             printf("%02x,",copy_buff[i]);
			 if(i == 0) {
			 	printf("\n");
			 }
          }
          printf(" \n\n");
		  
        printf("sif_spi_write_bytes recv:\n");
        for(i=0;i<total_num;i++)
          {
             printf("%02x,",tx_cmd[i]);
			 if(i == 0) {
			 	printf("\n");
			 }
          }
          printf(" \n\n");
        err_ret = -4;
         return err_ret;
    }

    pos = pos+CMD_RESP_SIZE+count+1;
    for(i=0;i<data_resp_size_w;i++)
    {
        if(tx_cmd[pos+i] == 0xE5)
        {
            for(i++;i<data_resp_size_w ;i++)
            {
                if( tx_cmd[pos+i] == 0xFF)
                {
                    break;
                }
            }
            break;      
        }
    }
    if(i>=data_resp_size_w)
    {
      printf("spierr byte write data no-busy wait byte 0xff no recv \n"); 
      for(i=0;i<total_num;i++)
      {
         printf("%02x,",tx_cmd[i]);
      }
      printf(" \n\n");
	  printf(" \n\n");
      
      for(i=0;i<total_num;i++)
      {
         printf("%02x,",copy_buff[i]);
      }
      printf(" \n\n");
      
      err_ret = -5;
      return err_ret;
    }
    
    return err_ret;
}


/**
 *	sdio_writesb - write to a FIFO of a SDIO function
 *	@func: SDIO function to access
 *	@addr: address of (single byte) 
 *	@src: buffer that contains the data to write
 *	@count: number of bytes to write
 *
 *	Writes to the specified FIFO of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */
 
int BusWriteBytes(u32 addr, void *src,u32 count)
{     
        int i;
        int pos,data_resp_size,total_num;       
        unsigned char *tx_data = (unsigned char*)src;
        int err_ret = 0;     
//printf("Byte Write ---------");  
        data_resp_size=((((((count>>2)+1) *25)>>5)*21+16)>>3)+1;/*((((((count>>5)+1) *75)) +44)>>3) +1,DATA_RESP_SIZE_W*/
         if(count > 512 )
        {
                err_ret = -1;
                return err_ret;
        }
   //      taskENTER_CRITICAL();
        tx_cmd[0]=0x75;

        if( addr >= (1<<17) )
        {
                err_ret = -2;
   //             taskEXIT_CRITICAL();
                return err_ret;

        }
        else
        {
             tx_cmd[1]=0x90|0x04|(addr>>15);    //0x94;
             tx_cmd[2]=addr>>7;    

             if(count == 512 )
            {
                    tx_cmd[3]=( addr<<1|0x0 );
                    tx_cmd[4]= 0x00;     //0x08;
            }
            else
            {
                    tx_cmd[3]=( addr<<1|count>>7 );
                    tx_cmd[4]= count;     //0x08;
            }
        }
        
        tx_cmd[5]=0x01;
        pos = 5+1;

//Add cmd respon
        memset(tx_cmd+pos,0xff,CMD_RESP_SIZE);
        pos =pos+ CMD_RESP_SIZE;

//Add token        
        tx_cmd[pos]=0xFE;
        pos = pos+1;

//Add data        
        memcpy(tx_cmd+pos,tx_data,count);
        pos = pos+count;

//Add data respon
        memset(tx_cmd+pos,0xff,data_resp_size);
        pos = pos+ data_resp_size;
        total_num=pos;
        memcpy(copy_buff,tx_cmd,total_num);
        STSpiWrite(tx_cmd,pos);
 
        pos = 5+1;
        for(i=0;i<CMD_RESP_SIZE;i++)
        {
                if(tx_cmd[pos+i] == 0x00)
                {
                        if(tx_cmd[pos+i+1] == 0x00)
                            break;      
                }
        }
        if(i>=CMD_RESP_SIZE)
        {
          printf("\n write cmd resp no recv\n");
          for(i=0;i<total_num;i++)
          {
             printf("%02x,",tx_cmd[i]);
          }
          printf(" \n\n");
          
          for(i=0;i<total_num;i++)
          {
             printf("%02x,",copy_buff[i]);
          }
          printf(" \n\n");
         
  //        taskEXIT_CRITICAL();
          return -3;
       }
        
        pos = pos+CMD_RESP_SIZE+count+1;
        for(i=0;i<data_resp_size;i++)
        {
          if(tx_cmd[pos+i] == 0xE5)
          {
                  break;      
          }
        }
        
        if(i>=data_resp_size)
        {
          printf("\n write data resp no recv:\n");
          for(i=0;i<total_num;i++)
          {
             printf("%02x,",tx_cmd[i]);
          }
          printf(" \n\n");
          for(i=0;i<total_num;i++)
          {
             printf("%02x,",copy_buff[i]);
          }
          printf(" \n\n");
          
   //       taskEXIT_CRITICAL();
          return -4;
        }
   //   taskEXIT_CRITICAL();
       return err_ret;
}

int sif_spi_read_bytes(u32 addr,u8* dst,u16 count,u8 func)
{      
    int pos,total_num,data_resp_size_r;
    int i;
    unsigned char *rx_data=dst;
    int err_ret = 0;
    int unexp_byte = 0;
    int find_start_token = 0;
    data_resp_size_r = ((((((count>>2)+1) *25)>>5)*21+16)>>3) +1+10; 

    rx_cmd[0]=0x75;

    if(count > 512 )
    {
        err_ret = -1;
        return err_ret;
    }

    if(func > 7)
    {
        err_ret = -3;
        return err_ret;
    }
    if( addr >= (1<<17) )
    {
        err_ret = -2;
        return err_ret;
    }
    else
    {
        rx_cmd[1]=(func<<4)|0x04|(addr>>15);    //0x14;
        rx_cmd[2]=addr>>7;    

        if(count == 512 )
        {
            rx_cmd[3]=( addr<<1|0x0 );
            rx_cmd[4]= 0x00;     //0x08;
        }
        else
        {
          rx_cmd[3]=( addr<<1|(count>>8 & 0x01) );
          rx_cmd[4]= count & 0xff;     //0x08;
  //          printf("rx[3,4]1:%02x,%02x\n",rx_cmd[3],rx_cmd[4]);
  //          rx_cmd[3]=( addr<<1|count>>7 );
  //          rx_cmd[4]= count;     //0x08;
    //        printf("rx[3,4]2:%02x,%02x\n",rx_cmd[3],rx_cmd[4]);
   //        rx_cmd[3]=( addr<<1|(count>>8 & 0x01) );
   //        rx_cmd[4]= count & 0xff; 
        }
    }

    rx_cmd[5]=0x01;

    total_num = CMD_RESP_SIZE+data_resp_size_r+count+2;
    memset(rx_cmd+6 , 0xFF ,total_num);
    total_num+=6;
    memcpy(copy_buff,rx_cmd,total_num);
    STSpiRead(rx_cmd,total_num);
		  
    pos = 5+1;
    for(i=0;i<CMD_RESP_SIZE;i++)
    {
        if(rx_cmd[pos+i] == 0x00 && rx_cmd[pos+i-1] == 0xff)
        {
            if(rx_cmd[pos+i+1] == 0x00 && rx_cmd[pos+i+2] == 0xff)
                break;      
        }
    }
    
    if(i>=CMD_RESP_SIZE)
    {
        printf("spierr byte read cmd resp 0x00 no recv\n");
        err_ret = -4;

		for(i=0;i<total_num;i++)
        {
        	printf("%02x ",copy_buff[i]);
			if((i%30) == 0) {
				printf("\n");
			}
        }   

		printf("\n\n");
		printf("\n\n");
		
		for(i=0;i<total_num;i++)
        {
        	printf("%02x ",rx_cmd[i]);
			if((i%30) == 0) {
				printf("\n");
			}
        }    
        return err_ret;
    }

    pos = pos+i+2;

    for(i=0;i<data_resp_size_r;i++)
    {
        if(rx_cmd[pos+i]==0xFE)
        {

            find_start_token = 1;   
            break;
        }
        else if(rx_cmd[pos+i] != 0xff)
        {
            unexp_byte ++;
            if(unexp_byte == 1)
                printf("1 normal byte read not 0xFF or 0xFE  at the first time,count = %d,addr =%x \n",count,addr);
        }
    }

    if(find_start_token == 0) 
    { 
      printf("spierr byte read start token 0xFE no recv ,count = %d,addr =%x \n",count,addr);

      err_ret = -5;

      return err_ret;
    }

    pos = pos+i+1;
   // memcpy(rx_data,rx_cmd+pos,count);
        for(i=0;i<count;i++)
        {
                *(rx_data+i) = rx_cmd[pos+i];
        }    
    return err_ret;
}

/**
 *	sdio_readsb - read from a FIFO on a SDIO function
 *	@func: SDIO function to access
 *	@dst: buffer to store the data
 *	@addr: address of (single byte) FIFO  addr:0~0xfffff
 *	@count: number of bytes to read        count<=512
 *
 *	Reads from the specified FIFO of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */  
int BusReadBytes(u32 addr, void *dst, u32 count)
{
        int err_ret = 0;
        int pos,total_num,data_resp_size;
        int i;
        unsigned char *rx_data = (unsigned char *)dst;
  
        data_resp_size=((((((count>>2)+1) *25)>>5)*21+16)>>3)+1;/*DATA_RESP_SIZE_R*/
   //     taskENTER_CRITICAL();
        rx_cmd[0]=0x75;
        
        if(count > 512 )
        {
            err_ret = -1;
   //         taskEXIT_CRITICAL();
            return err_ret;
        }

        if( addr >= (1<<17) )
        {
                err_ret = -2;
   //             taskEXIT_CRITICAL();
                return err_ret;
        }
        else
        {
             rx_cmd[1]=0x10|0x04|(addr>>15);    //0x14;
             rx_cmd[2]=addr>>7;    

             if(count == 512 )
            {
                    rx_cmd[3]=( addr<<1|0x0 );
                    rx_cmd[4]= 0x00;     //0x08;
            }
            else
            {
                    rx_cmd[3]=( addr<<1|count>>7 );
                    rx_cmd[4]= count;     //0x08;
            }
        }
        
        rx_cmd[5]=0x01;
        total_num = CMD_RESP_SIZE+data_resp_size+count+2;
        memset(rx_cmd+6 , 0xFF ,total_num);
        total_num+=6; 
        memcpy(copy_buff,rx_cmd,total_num);
        STSpiRead(rx_cmd,total_num);

//printf("mark 2",pos+i);
        pos = 5+1;
        for(i=0;i<CMD_RESP_SIZE;i++)
        {
                if(rx_cmd[pos+i]==0x00)
                {
                    break;
                }
        }
        if(i>=CMD_RESP_SIZE)
        {
          printf("\n read cmd resp no recv:\n");
          for(i=0;i<total_num;i++)
          {
             printf("%02x,",rx_cmd[i]);
          }
          printf(" \n\n");
          
          for(i=0;i<total_num;i++)
          {
             printf("%02x,",copy_buff[i]);
          }
          printf(" \n\n");
          
    //      taskEXIT_CRITICAL();
          return -3;
        }

       pos = pos+i+2;
       
        for(i=0;i<data_resp_size;i++)
        {
          if(rx_cmd[pos+i]==0xFE)
          {
              break;
          }
        }
        if(i>=data_resp_size)
        {
          printf("\n read data resp no recv:\n");
          for(i=0;i<(total_num+6);i++)
          {
             printf("%02x,",rx_cmd[i]);
          }
          printf(" \n\n");
          for(i=0;i<total_num;i++)
          {
             printf("%02x,",copy_buff[i]);
          }
          printf(" \n\n");
   //       taskEXIT_CRITICAL();
          return -4;
       }

       pos = pos+i+1;

        for(i=0;i<count;i++)
        {
                *(rx_data+i) = rx_cmd[pos+i];
        }    
   //     taskEXIT_CRITICAL();
        return err_ret;
}

int sif_spi_write_blocks(u32 addr,u8* src, u16  count,u16 block_size)
{
    int err_ret = 0;
    int i,j;
    int n;
    int pos; 
    unsigned char *tx_data = src;
    int find_w_rsp = 0;

    tx_cmd[0]=0x75;

    if( count <=0 )
    {
        err_ret = -1;
        return err_ret;
    }

    if( addr >= (1<<17) )
    {
        err_ret = -2;
        return err_ret;
    }
    else
    {
        tx_cmd[1]=0x90|0x0C|(addr>>15);   
        tx_cmd[2]=addr>>7;    

        if(count >= 512 )
        {
            tx_cmd[3]=( addr<<1|0x0 );
            tx_cmd[4]= 0x00;     
        }
        else
        {
            tx_cmd[3]=( addr<<1|(count>>8 & 0x01) );
            tx_cmd[4]= count & 0xff ;     
        }
    }
    tx_cmd[5]=0x01;

    pos = 5+1;
    //Add cmd respon
    memset(tx_cmd+pos,0xff,CMD_RESP_SIZE);
    pos =pos+ CMD_RESP_SIZE;

    for(j=0;j<count;j++)
    {
        //Add token   
        tx_cmd[pos]=0xFC;
        pos = pos+1;
        //Add data    
        memcpy(tx_cmd+pos,tx_data+j*block_size, block_size);
        pos = pos+block_size;
        //Add data respon
        if( j==(count- 1))
        {
            memset(tx_cmd+pos , 0xff ,  BLOCK_W_DATA_RESP_SIZE_FINAL );
            pos = pos+  BLOCK_W_DATA_RESP_SIZE_FINAL;
            continue;
        }
        memset(tx_cmd+pos , 0xff , BLOCK_W_DATA_RESP_SIZE_EACH);
        pos = pos+ BLOCK_W_DATA_RESP_SIZE_EACH;
    }

    STSpiWrite(tx_cmd,pos);

    //Judge Write cmd resp, and 1st block data resp.        
    pos = 5+1;
    for(i=0;i<CMD_RESP_SIZE;i++)
    {
        if(tx_cmd[pos+i] == 0x00 && tx_cmd[pos+i-1] == 0xff)
        {
            if(tx_cmd[pos+i+1] == 0x00 && tx_cmd[pos+i+2] == 0xff)
                break;      
        }
    }

    if(i>=CMD_RESP_SIZE)
    {
        printf("spierr 1st block write cmd resp 0x00 no recv, %d\n", count);
        err_ret = -3;
        return err_ret;
    }

    pos = pos+CMD_RESP_SIZE;

    for(j=0;j<count;j++)
    {
        find_w_rsp = 0;
        pos = pos+1;                   
        pos = pos+block_size;

        if( j==(count-1) )
            n = BLOCK_W_DATA_RESP_SIZE_FINAL;
        else
            n= BLOCK_W_DATA_RESP_SIZE_EACH;

        for(i =0 ;i<n;i++)
        {
            if((tx_cmd[pos+i] & 0x0F) == 0x05)
            {
                find_w_rsp = 1;
                break;
            }
        }

        if(find_w_rsp == 0)
        {
            err_ret = -6;
            printf( "spierr %s block%d write data no data res error, %d\n", __func__, j+1, count);
            return err_ret;
        }
        pos = pos+n;     
    }
    return err_ret;
}
/**
 *	sdio_memcpy_toio - write a chunk of memory to a SDIO function
 *	@func: SDIO function to access
 *	@addr: address to start writing to
 *	@src: buffer that contains the data to write
 *	@count: number of blocks to write
 *
 *	Writes to the address space of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */
int BusWriteBlocks(u32 addr,void *src, u32 count)
{
	 int err_ret = 0;
        int i,j;
        int n;
        int pos; 
        unsigned char *tx_data = (unsigned char*)src;
//if(count>12)
 //       printf("blk cnt:%d\n",count);
#if 0
GPIO_SetBits(GPIOB, GPIO_Pin_12);
        for(j=0;j<count;j++)
                Memcpy(tx_cmd,tx_data+j*SPI_BLOCK_SIZE,SPI_BLOCK_SIZE);
        
        GPIO_ResetBits(GPIOB, GPIO_Pin_12);
        return 0;

#endif

#if 0       //Add here for seq no checking.
unsigned int seq[13] = {0};
unsigned short cur_len ;

char t;
unsigned int temp_len = 0;
for(t=0;t<13;t++)
{
       seq[t]= *(unsigned int *)(tx_data+temp_len+8);
       cur_len = *(unsigned short *)(tx_data+temp_len+2);
       //temp_len += *(unsigned short *)(tx_data+temp_len+2);
       temp_len += (cur_len%260)!=0 ? (cur_len/260+1)*260:cur_len ;
}
printf("seq:%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",seq[0],seq[1],seq[2],seq[3],seq[4],
                 seq[5],seq[6],seq[7],seq[8],seq[9],seq[10],seq[11],seq[12]);
#endif
        
#if 1

//GPIO_SetBits(GPIOB, GPIO_Pin_12);      
//GPIO_ResetBits(GPIOB, GPIO_Pin_12);   
//printf("Block Write ---------");    
        tx_cmd[0]=0x75;

        if( count <=0 )
        {
                err_ret = -1;
                return err_ret;
        }

        if( addr >= (1<<17) )
        {
                err_ret = -2;
                return err_ret;
        }
        else
        {
             tx_cmd[1]=0x90|0x0C|(addr>>15);   
             tx_cmd[2]=addr>>7;    

             if(count >= 512 )
            {
                    tx_cmd[3]=( addr<<1|0x0 );
                    tx_cmd[4]= 0x00;     
            }
            else
            {
                    tx_cmd[3]=( addr<<1|count>>7 );
                    tx_cmd[4]= count;     
            }
        }
        tx_cmd[5]=0x01;
        
        pos = 5+1;

//Add cmd respon
        memset(tx_cmd+pos,0xff,CMD_RESP_SIZE);
        pos =pos+ CMD_RESP_SIZE;

         for(j=0;j<count;j++)
         {
//Add token   
                tx_cmd[pos]=0xFC;
                pos = pos+1;
//Add data        
                //for(n=0;n<SPI_BLOCK_SIZE;n++)
                        //tx_cmd[pos+n] = *(tx_data+j*SPI_BLOCK_SIZE+n);
                Memcpy(tx_cmd+pos,tx_data+j*SPI_BLOCK_SIZE,SPI_BLOCK_SIZE);
                
                pos = pos+SPI_BLOCK_SIZE;
//Add data respon
                if( j==(count-1) )
                {
                        memset(tx_cmd+pos , 0xff , BLOCK_W_DATA_RESP_SIZE_FINAL);
                        pos = pos+ BLOCK_W_DATA_RESP_SIZE_FINAL;
                        continue;
                }
                memset(tx_cmd+pos , 0xff , BLOCK_W_DATA_RESP_SIZE_EACH);
                pos = pos+ BLOCK_W_DATA_RESP_SIZE_EACH;
         }
         
//GPIO_SetBits(GPIOB, GPIO_Pin_12);
//GPIO_ResetBits(GPIOB, GPIO_Pin_12);

        STSpiWrite(tx_cmd,pos);
        //STSpiWrite(tx_data,1500);

//GPIO_ResetBits(GPIOB, GPIO_Pin_12);
//GPIO_SetBits(GPIOB, GPIO_Pin_12);

//Judge Write cmd resp, and 1st block data resp.        
        pos = 5+1;
        for(i=0;i<CMD_RESP_SIZE;i++)
        {
                if(tx_cmd[pos+i] == 0x00)
                {
#ifdef test
printf("0x00 pos:%d\n",i);  
#endif
                        if(tx_cmd[pos+i+1] == 0x00)
                            break;      
                }
        }
        if(i>=CMD_RESP_SIZE)
        {
                printf("1st block write cmd resp 0x00 no recv\n");
                //kfree(tx_cmd); 
                return -3;
       }

        pos = pos+CMD_RESP_SIZE;
        
        for(j=0;j<count;j++)
        {
//Judge block data resp
                pos = pos+1;                   
                pos = pos+SPI_BLOCK_SIZE;
                
                if( j==(count-1) )
                    n = BLOCK_W_DATA_RESP_SIZE_FINAL;
                else
                    n= BLOCK_W_DATA_RESP_SIZE_EACH;
                
                for(i=0;i<n;i++)
                {
                        //if(tx_cmd[pos+i] == 0xE5)
                        if( (tx_cmd[pos+i]&0x0F) == 0x05 )
                        {
#ifdef test                        
printf("%d block ,0xE5 pos:%d\n",j,i);
#endif
                                for(i++;i<n;i++)
                                {
                                     if( tx_cmd[pos+i] == 0xFF)
                                     {
#ifdef test                
                                            if(i>max_block_dataW_resp_size)
                                             {
                                                    max_block_dataW_resp_size = i;
                                                    printf("new block data write MAX 0xFF pos:%d\n",max_block_dataW_resp_size);   
                                             }
printf("%d block ,0xFF pos:%d\n",j,i);
#endif
                                          break;
                                     }
                                }
                                break;     
                        }
                }
                if(i>=n)
                {
                        printf("totoal:%d, block %d write data resp 0xE5 no recv\n",count, j+1);
                        //kfree(tx_cmd); 
                        return -4;
               }
                pos = pos+n;   
        }
        
//GPIO_SetBits(GPIOB, GPIO_Pin_12);      
//GPIO_ResetBits(GPIOB, GPIO_Pin_12);   

#endif
       return err_ret;
}


int sif_spi_read_blocks(u32 addr, u8 *dst, u16 count,u16 block_size)
{
    int err_ret = 0;
    int pos;
    int i,j;
    unsigned char *rx_data =dst;
    int total_num;
    int find_start_token = 0;
    
    rx_cmd[0]=0x75;

    if( count <=0 )
    {
        err_ret = -1;
        return err_ret;
    }
    if( addr >= (1<<17) )
    {
        err_ret = -2;
        return err_ret;
    }
    else
    {
        rx_cmd[1]=0x10|0x0C|(addr>>15);   
        rx_cmd[2]=addr>>7;    

        if(count >= 512 )
        {
            rx_cmd[3]=( addr<<1|0x0 );
            rx_cmd[4]= 0x00;     
        }
        else
        {
            rx_cmd[3]=( addr<<1|(count>>8 & 0x01) );
            rx_cmd[4]= count & 0xff;     
        }
    }
    rx_cmd[5]=0x01;
    total_num = CMD_RESP_SIZE+BLOCK_R_DATA_RESP_SIZE_1ST+block_size+ 2 + (count-1)*(BLOCK_R_DATA_RESP_SIZE_EACH+block_size+2);
    memset(rx_cmd+6, 0xFF ,total_num);
    total_num+=6;      
    STSpiRead(rx_cmd,total_num );

    pos = 5+1;
    for(i=0;i<CMD_RESP_SIZE;i++)
    {
        if(rx_cmd[pos+i] == 0x00 && rx_cmd[pos+i-1] == 0xff)
        {
            if(rx_cmd[pos+i+1] == 0x00 && rx_cmd[pos+i+2] == 0xff)
                break;      
        }
    }

    if(i>=CMD_RESP_SIZE)
    {
         printf("spierr block read cmd resp 0x00 no recv\n");

        err_ret = -3;
        return err_ret;
    }

    pos = pos+i+2;

    for(i=0;i<BLOCK_R_DATA_RESP_SIZE_1ST;i++)
    {
        if(rx_cmd[pos+i]==0xFE)
        {
            find_start_token = 1;
            break;
        }
    }

    if( find_start_token == 0) 
    {       
         printf("1st block read data resp 0xFE no recv,count = %d\n",count);  
        err_ret = -4;            
        return err_ret; 
    }

    pos = pos+i+1;

    memcpy(rx_data,rx_cmd+pos,block_size);

    pos = pos +block_size + 2;

    for(j=1;j<count;j++)
    {

        for(i=0;i<BLOCK_R_DATA_RESP_SIZE_EACH;i++)
        {
            if(rx_cmd[pos+i]==0xFE)
            {
                break;
            }
        }
        if(i>=BLOCK_R_DATA_RESP_SIZE_EACH)
        {
            printf("spierr block%d read data token 0xFE no recv,total:%d\n",j+1,count);
            err_ret = -7;
            return err_ret; 
        }
        pos = pos+i+1;

        memcpy(rx_data+j*block_size,rx_cmd+pos,block_size);
        pos = pos +block_size + 2;
    }
    return err_ret;
}

/**
 *	sdio_memcpy_fromio - read a chunk of memory from a SDIO function
 *	@func: SDIO function to access
 *	@dst: buffer to store the data
 *	@addr: address to begin reading from
 *	@count: number of bytes to read
 *
 *	Reads from the address space of a given SDIO function. Return
 *	value indicates if the transfer succeeded or not.
 */
int BusReadBlocks(u32 addr, void *dst, u32 count)
{
       int err_ret = 0;
       int pos;
       int i,j;
       unsigned char *rx_data = (unsigned char *)dst;
       int total_num;

#if 0
        GPIO_SetBits(GPIOB, GPIO_Pin_12);
        for(j=0;j<count;j++)
                Memcpy(rx_data+j*SPI_BLOCK_SIZE, rx_cmd,SPI_BLOCK_SIZE);

        
        GPIO_ResetBits(GPIOB, GPIO_Pin_12);
        return 0;
#endif

//GPIO_SetBits(GPIOB, GPIO_Pin_12);      
//GPIO_ResetBits(GPIOB, GPIO_Pin_12);  
       
//if(count>12)
    //printf("%d\n",count);
    
//printf("Block Read ---------"); 

        rx_cmd[0]=0x75;

        if( count <=0 )
        {
                err_ret = -1;
                return err_ret;
        }
        if( addr >= (1<<17) )
        {
                err_ret = -2;
                return err_ret;
        }
        else
        {
             rx_cmd[1]=0x10|0x0C|(addr>>15);   
             rx_cmd[2]=addr>>7;    

            if(count >= 512 )
            {
                    rx_cmd[3]=( addr<<1|0x0 );
                    rx_cmd[4]= 0x00;     
            }
            else
            {
                    rx_cmd[3]=( addr<<1|count>>7 );
                    rx_cmd[4]= count;     
            }
        }
        rx_cmd[5]=0x01;
        total_num = CMD_RESP_SIZE+BLOCK_R_DATA_RESP_SIZE_1ST+SPI_BLOCK_SIZE+ 2 + (count-1)*(BLOCK_R_DATA_RESP_SIZE_EACH+SPI_BLOCK_SIZE+2);
        memset(rx_cmd+6 , 0xFF ,total_num);

//GPIO_SetBits(GPIOB, GPIO_Pin_12);      
//GPIO_ResetBits(GPIOB, GPIO_Pin_12);  


        STSpiRead(rx_cmd,6+total_num );

//GPIO_SetBits(GPIOB, GPIO_Pin_12);      
//GPIO_ResetBits(GPIOB, GPIO_Pin_12);  


        pos = 5+1;
        for(i=0;i<CMD_RESP_SIZE;i++)
        {
                if(rx_cmd[pos+i]==0x00)
                {
#ifdef test                
printf("0x00 pos:%d\n",i);
#endif
                    break;
                }
        }
        if(i>=CMD_RESP_SIZE)
        {
                printf("block read cmd resp 0x00 no recv\n");
                //kfree(rx_cmd);
                return -3;
        }

        pos = pos+i+2;

        for(i=0;i<BLOCK_R_DATA_RESP_SIZE_1ST;i++)
        {
                if(rx_cmd[pos+i]==0xFE)
                {
#ifdef test                
                        if(i>max_block_dataR_resp_size)
                         {
                                max_block_dataR_resp_size = i;
                                printf("new block data read MAX 0xFE pos:%d\n",max_block_dataR_resp_size);   
                         }
printf("0xFE pos:%d\n",i);
#endif
                    break;
                }
        }
        if(i>=BLOCK_R_DATA_RESP_SIZE_1ST)
        {
                printf("1st block read data resp 0xFE no recv\n");
                return -4;
        }

        pos = pos+i+1;

        for(i=0;i<SPI_BLOCK_SIZE;i++)
        {
                *(rx_data+i) = rx_cmd[pos+i];
        }
        
        pos = pos +SPI_BLOCK_SIZE + 2;

        for(j=1;j<count;j++)
        {
                for(i=0;i<BLOCK_R_DATA_RESP_SIZE_EACH;i++)
                {
                        if(rx_cmd[pos+i]==0xFE)
                        {
#ifdef test                
                                if(i>max_block_dataR_resp_size)
                                 {
                                        max_block_dataR_resp_size = i;
                                        printf("new block data read MAX 0xFE pos:%d\n",max_block_dataR_resp_size);   
                                 }
printf("0xFE pos:%d\n",i);
#endif
                            break;
                        }
                }
                if(i>=BLOCK_R_DATA_RESP_SIZE_EACH)
                {
                        printf("total:%d, block %d read data resp 0xFE no recv\n",count,j+1);
                        return -4;
                }
//printf("rest_num:%d",rest_num);
                pos = pos+i+1;
                /*
                for(i=0;i<SPI_BLOCK_SIZE;i++)
                {
                        *(rx_data+j*SPI_BLOCK_SIZE+i) = rx_cmd[pos+i];
                }
                */
                Memcpy(rx_data+j*SPI_BLOCK_SIZE, rx_cmd+pos,SPI_BLOCK_SIZE);
                
                pos = pos +SPI_BLOCK_SIZE + 2;
                
        }

//GPIO_SetBits(GPIOB, GPIO_Pin_12);      
//GPIO_ResetBits(GPIOB, GPIO_Pin_12);  

      return err_ret;
}

#endif
