#ifndef EGL_SYS_ARCH_H
#define EGL_SYS_ARCH_H

#include "sip2_common.h"
#include "port/port_arch.h"

/*
Note: Please make your platform port_arch.c , port_arch.h  to realize the func below
*/


#define SYS_ARCH_TIMEOUT 0xffffffffUL

/** sys_mbox_tryfetch() returns SYS_MBOX_EMPTY if appropriate.
 * For now we use the same magic value, but we allow this to change in future.
 */
#define SYS_MBOX_EMPTY SYS_ARCH_TIMEOUT 

typedef void (*egl_thread_fn)(void *arg);
typedef void (*egl_timer_fn)( egl_timer_t pxTimer );

int egl_mutex_new(egl_mutex_t *mutex);
void egl_mutex_lock(egl_mutex_t *mutex);
void egl_mutex_unlock(egl_mutex_t *mutex);
void egl_mutex_free(egl_mutex_t *mutex); 
#ifndef egl_mutex_valid
int egl_mutex_valid(egl_mutex_t *mutex);
#endif
#ifndef egl_mutex_set_invalid
void egl_mutex_set_invalid(egl_mutex_t *mutex);
#endif

int egl_sem_new(egl_sem_t *sem, u8 count);          //Binary Sem
int egl_cntsem_new(egl_sem_t *sem, u8 count);       //Counting Sem
void egl_sem_signal(egl_sem_t *sem);
u32 egl_arch_sem_wait(egl_sem_t *sem, u32 timeout);
void egl_sem_free(egl_sem_t *sem);
#define egl_sem_wait(sem)                  egl_arch_sem_wait(sem, 0)


int egl_mbox_new(egl_mbox_t *mbox, int size);

void egl_mbox_post(egl_mbox_t *mbox, void *msg);

int egl_mbox_trypost(egl_mbox_t *mbox, void *msg);

u32 egl_arch_mbox_fetch(egl_mbox_t *mbox, void **msg, u32 timeout);

#ifndef egl_arch_mbox_tryfetch
u32 egl_arch_mbox_tryfetch(egl_mbox_t *mbox, void **msg);
#endif

#define egl_mbox_tryfetch(mbox, msg) egl_arch_mbox_tryfetch(mbox, msg)

void egl_mbox_free(egl_mbox_t *mbox);
#define egl_mbox_fetch(mbox, msg) egl_arch_mbox_fetch(mbox, msg, 0)


#ifndef egl_mbox_valid
int egl_mbox_valid(egl_mbox_t *mbox);
#endif
#ifndef egl_mbox_set_invalid
void egl_mbox_set_invalid(egl_mbox_t *mbox);
#endif


/* egl_init() must be called before anthing else. 
initilize:  bus_host mutex: BusHostMutex.
*/
void egl_init(void);

egl_thread_t egl_thread_new(const char *name, egl_thread_fn thread, void *arg, int stacksize, int prio);
//void egl_thread_enter_critical(void);  
//void egl_thread_exit_critical(void);  


void egl_thread_set_prio(egl_thread_t task, int prio);      
int egl_thread_get_prio(egl_thread_t task);   
void egl_thread_del(egl_thread_t thread);

//Not necessary
void egl_thread_yield(void);

/*
egl_timer_t egl_timer_new(const signed char *name, egl_timer_fn timer, unsigned long ticks);
int egl_timer_start(egl_timer_t timer);
int egl_timer_stop(egl_timer_t timer);
int egl_timer_del(egl_timer_t timer);
*/
    
u32 egl_now(void);
void egl_delay(portTickType xTicksToDelay );


void *egl_malloc(u32 size );
void egl_free(void *bufPtr );

#endif /* __EGL_SYS_H__ */

