#include <stdio.h>
#include "bus/egl_bus_hw.h"
#include "sys/egl_sys_arch.h"

#include "port/wifi_user_interface.h"
#include "esp_main_sim.h"

#include "usart_cfg.h"
#include "stm32f10x.h"
#include "esp_pub.h"

u32 SPI_BLOCK_SIZE = 512;

#define BUFFERSIZE 2048

#define RX_AVAILIBLE 2
#define TX_AVAILIBLE 1
#define INIT_STAGE   0

struct sdio_slave_status_element
{
    u32 wr_busy:1;
    u32 rd_empty :1;
    u32 comm_cnt :3;
    u32 intr_no :3;
    u32 rx_length:16;
    u32 res:8;
};

extern egl_sem_t  QueueWorkCntSem ;
u8 tx_buff[10*1024],rx_buff[BUFFERSIZE];

u8 sdio_comm_cnt;
xTimerHandle test_timer;
u32 total_recv_data_len = 0;
void test_timer_cb( xTimerHandle xTimer )
{
  printf("recv %d bytes\r\n",total_recv_data_len);
  total_recv_data_len = 0;
}
int tx_len = 0;



void esp_printf_put_char(u8 ch)
{
#if (PRINTF_UART_METHOD==0)
               //USART_ClearFlag(USART1, USART_FLAG_TC);
               while (USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET);
               USART_SendData(USART1, (uint8_t)ch);   
                /* Loop until the end of transmission */
               while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
#else
               xSerialPutChar( ch, 0xFFFFFFFF );
#endif
}
#if SPI_SUPPORT
void SdioRW(void *pvParameters)
{

    int i,ret;
    u32 prt_cnt=0,intr_clr; /*power_temp,*/
    struct sdio_slave_status_element sd_sta;
    u8 rd_not_wr=0;
    egl_delay(3000);
    printf("SDIO Communication thread created \n ");
#if 1    
    SPI_BLOCK_SIZE = 2048;
    {
      unsigned short blk_size=SPI_BLOCK_SIZE,xx;
      for(xx=0;xx<10000;xx++);
      ret=sif_spi_write_bytes(0x110,(u8*)&blk_size,2,0);
      for(xx=0;xx<10000;xx++);
    }
#endif    
    test_timer = xTimerCreate("test",(60*1000)/portTICK_RATE_MS,1,NULL,test_timer_cb);
    ret=sif_spi_read_bytes(0x20, (u8 *)&sd_sta, 4,1);
    //ret=BusReadBytes(0x20, (void *)&sd_sta, 4);
    if(ret)
    {
        printf("Init Read reg err:-0x%x\n ",-ret);
    }
    sdio_comm_cnt=sd_sta.comm_cnt;
xTimerStart(test_timer,0);
    while(1)
    {

        egl_arch_sem_wait(&BusIrqReadSem,0);
		//egl_delay(10);
        //   ret=BusReadBytes(0x20, (void *)&sd_sta, 4);
        ret=sif_spi_read_bytes(0x20, (u8 *)&sd_sta, 4,1);
        if(ret)
        {
            printf("Read reg err:-0x%x\n ",-ret);
			continue;
        }
        intr_clr=1;
        //   ret=BusWriteBytes(0x30, (void *)&intr_clr, 4);
        ret=sif_spi_write_bytes(0x30, (u8*)&intr_clr, 4,1);
        if(ret)
        {
            printf("Write reg err:-0x%x\n ",-ret);
			continue;
        }
		//printf("\n\nreg:%d,%d,%d,%d,%d\n ",sd_sta.intr_no,sd_sta.rd_empty,sd_sta.rx_length,sdio_comm_cnt,sd_sta.comm_cnt);

		if((!sd_sta.rd_empty)
           //&& (sdio_comm_cnt==sd_sta.comm_cnt)
           && (sd_sta.rx_length != 0))
        {            
            // ret=BusReadBytes(0x1f800-sd_sta.rx_length, (void *)rx_buff, sd_sta.rx_length); // OK
            int len = sd_sta.rx_length;
            u8* ptr_buf = rx_buff;
            if(len >= SPI_BLOCK_SIZE) {
		ret=sif_spi_read_blocks(0x1f800 - len, (u8 *)ptr_buf, len/SPI_BLOCK_SIZE,SPI_BLOCK_SIZE);
		if(ret)
	        {
	           printf("Read first Data err:-0x%x\n",-ret);
                   continue;
	        }
		ptr_buf += sd_sta.rx_length - len%SPI_BLOCK_SIZE;
                total_recv_data_len += sd_sta.rx_length - len%SPI_BLOCK_SIZE;
		len = len%SPI_BLOCK_SIZE;
            } 

            while ((len >= 512) && (ret == 0)) {
              ret=sif_spi_read_bytes(0x1f800- len, (u8 *)ptr_buf, 512,1);
              len -= 512;
              ptr_buf += 512;
              if(ret)
              {
                printf("Read Data second err:-0x%x,%d\n",-ret,len);
              } else {
                total_recv_data_len += 512;
              }
            }
            
            if((len != 0) && (ret == 0)) {
		ret=sif_spi_read_bytes(0x1f800- len, (u8 *)ptr_buf, len,1);
                if(ret == 0) {
                  total_recv_data_len += len;
                }
            }
            
            rx_buff[sd_sta.rx_length] = '\0';
            if(ret)
            {
                printf("Read Data err:-0x%x,%d\n",-ret,len);
            } else {
                int loop = 0;
#if 0
                for (loop = 0; loop < sd_sta.rx_length;loop++) {
                  xSerialPutChar( rx_buff[loop], 0xFFFFFFFF );
                }
#endif
                for (loop = 0; loop < sd_sta.rx_length;loop++) {
                  esp_printf_put_char(rx_buff[loop] );
                }
                //printf("%s",rx_buff);
		sdio_comm_cnt=(sdio_comm_cnt+1)&0x07;
            }

        }
		
        if(tx_len != 0)
        {
            //if((!sd_sta.wr_busy) && (sdio_comm_cnt==sd_sta.comm_cnt))
            if(!sd_sta.wr_busy)
            {                
                USART_ITConfig(USART1,USART_IT_RXNE,DISABLE);
                //ret=BusWriteBytes(0x1f800-tx_len, (void *)tx_buff, tx_len);  // OK
                int len = tx_len;
		u8* ptr_buf = tx_buff;
	            if(len >= SPI_BLOCK_SIZE) {
			ret=sif_spi_write_blocks(0x1f800-len, (u8 *)ptr_buf, len/SPI_BLOCK_SIZE,SPI_BLOCK_SIZE);
			if(ret)
		        {
                          printf("Read Data err:-0x%x\n",-ret);
                          continue;
		        }

					ptr_buf += tx_len - len%SPI_BLOCK_SIZE;
					len = len%SPI_BLOCK_SIZE;
	            }
				if(len != 0) {
					ret=sif_spi_write_bytes(0x1f800- len, (u8 *)ptr_buf, len,1);
				}
                tx_len = 0;
                USART_ITConfig(USART1,USART_IT_RXNE,ENABLE);

                //ret=sif_spi_write_bytes(0x1f800-tx_len, (u8 *)tx_buff, tx_len);
                //ret=sif_spi_write_blocks(0x1f800-tx_len, (u8*)tx_buff, blk_no,blk_size);
                if(ret)
                {
                    printf("Write Data err:-0x%x\n ",-ret);
                } else {
                	sdio_comm_cnt=(sdio_comm_cnt+1)&0x07;
                }
            }
        }        
    }
    printf("SDIO Communication thread delete\n");
    egl_thread_del(NULL);
}
#endif

#if SDIO_SUPPORT
u8 uart_tmp_buf[BUFFERSIZE];
u16 uart_tmp_len;

void SdioRW(void *pvParameters)
{
	extern void *EpubAddr;
	struct esp_pub *epub = (struct esp_pub *)EpubAddr;
	struct esp_sdio_ctrl *sctrl = epub->sif;
    int ret;
    struct sdio_slave_status_element sd_sta;
    printf("SDIO Communication thread created \n ");

    SDIO_SetSDIOOperation(ENABLE);
    egl_delay(1000);
#if 0
    SPI_BLOCK_SIZE = 2048;
    {
      unsigned short blk_size=SPI_BLOCK_SIZE,xx;
      //for(xx=0;xx<10000;xx++);
      //ret=sif_spi_write_bytes(0x110,(u8*)&blk_size,2,0);
      CMD52Write(0,0x110,0);
      CMD52Write(0,0x111,8);
      for(xx=0;xx<10000;xx++);
    }
#endif
    test_timer = xTimerCreate("test",(60*1000)/portTICK_RATE_MS,1,NULL,test_timer_cb);

    BusWriteReg(0x30,1);
    ret = BusReadBytes(0x20, (u8 *)&sd_sta, 4);
    if(ret)
    {
        printf("Init Read reg err:-0x%x\n ",-ret);
    }
    sdio_comm_cnt=sd_sta.comm_cnt;
	
	printf("\n\nfirst reg:%d,%d,%d,%d,%d,wr_busy %d\n ",sd_sta.intr_no,sd_sta.rd_empty,sd_sta.rx_length,sdio_comm_cnt,sd_sta.comm_cnt,sd_sta.wr_busy);
	xTimerStart(test_timer,0);

    while(1)
    {
		BusWriteReg(0x30,1);
        SDIO_ITConfig(SDIO_IT_SDIOIT, ENABLE);
        egl_arch_sem_wait(&BusIrqReadSem,0);
        //BusWriteReg(0x30,1);
        //ret=BusReadBytes (0x20, (u8 *)&sd_sta, 4);
        ret=BusReadBlocks(0x20, (u8 *)rx_buff, 1);
        memcpy(&sd_sta,rx_buff,sizeof(sd_sta));

        //ret = BusReadBytes(0x20, (u8 *)&sd_sta, 4);
        if(ret)
        {
            printf("Read reg err:-0x%x\n ",-ret);
			continue;
        }

        //printf("\n\nreg:%d,%d,%d,%d,%d,wr_busy %d\n ",sd_sta.intr_no,sd_sta.rd_empty,sd_sta.rx_length,sdio_comm_cnt,sd_sta.comm_cnt,sd_sta.wr_busy);
		if((!sd_sta.rd_empty)
           //&& (sdio_comm_cnt==sd_sta.comm_cnt)
           && (sd_sta.rx_length != 0))
        {            
            u32 read_len = 0;
            read_len = roundup(sd_sta.rx_length, sctrl->slc_blk_sz);
            read_len = read_len/(sctrl->slc_blk_sz);
            //printf("len:%d,%d\r\n",read_len,sd_sta.rx_length);
            ret=BusReadBlocks(0x1f800 - sd_sta.rx_length, (u8 *)rx_buff, read_len);

            if(ret)
            {
		printf("Read first Data err:-0x%x\n",-ret);
	        continue;
            }
	    total_recv_data_len += sd_sta.rx_length;
            
            rx_buff[sd_sta.rx_length] = '\0';
            if(ret)
            {
                printf("Read Data err:-0x%x,%d\n",-ret,sd_sta.rx_length);
            } else {
                int loop = 0;
//              if (sd_sta.rx_length < 200) {
                for (loop = 0;loop < sd_sta.rx_length;loop++) {
                  esp_printf_put_char(rx_buff[loop]);
                }
//              }
		sdio_comm_cnt=(sdio_comm_cnt+1)&0x07;
            }

        }

        if(tx_len != 0)
        {
            if(!sd_sta.wr_busy)
            {                
                u32 write_len = 0;
                USART_ITConfig(USART1,USART_IT_RXNE,DISABLE);
		if (tx_len <= BUFFERSIZE) {
                  uart_tmp_len = tx_len;
		} else {
                  uart_tmp_len = BUFFERSIZE;
                  xSemaphoreGive(BusIrqReadSem);
		}
                memcpy(uart_tmp_buf,tx_buff,uart_tmp_len);
		tx_len -= uart_tmp_len;
		if (tx_len > 0) {
                  memcpy(tx_buff,tx_buff + uart_tmp_len,tx_len);
		}
		USART_ITConfig(USART1,USART_IT_RXNE,ENABLE);
		//printf("len %d\r\n",tx_len);
                write_len = roundup(uart_tmp_len, sctrl->slc_blk_sz);
                write_len = write_len/(sctrl->slc_blk_sz);
                //printf("write_len %d\r\n",write_len);
		ret=BusWriteBlocks(0x1f800-uart_tmp_len, (u8 *)uart_tmp_buf,write_len);

                if(ret)
                {
                    printf("Write Data err:-0x%x\n ",-ret);
                } else {
                    sdio_comm_cnt=(sdio_comm_cnt+1)&0x07;
                }
            }
        } 
        
    }
    printf("SDIO Communication thread delete\n");
    egl_thread_del(NULL);
}
#endif


void SPITest(void)
{
    esp_sdio_probe();

    egl_thread_new( /*( signed portCHAR * )*/ "SDIOCOMM", SdioRW, NULL, 512, 12) ;
    egl_thread_del( NULL );
}

void ThreadMainProcess(void *pvParameters)
{
    extern void uartRxTask( void *pvParameters );
    printf("Eagle SPI Main thread created \n ");
	xTaskCreate( uartRxTask, ( signed char * ) "COMRx", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 2, NULL );
    bus_ini();

    SPITest();
}



