/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Simon Goldschmidt
 *
 */
#ifndef __LWIPOPTS_H__
#define __LWIPOPTS_H__

/*
   -----------------------------------------------
   ---------- Platform specific locking ----------
   -----------------------------------------------
*/
#define SYS_LIGHTWEIGHT_PROT        1
#define NO_SYS                                  0

/*
   ------------------------------------
   ---------- Memory options ----------
   ------------------------------------
*/
#define MEM_LIBC_MALLOC        0 
        /**
         * MEM_ALIGNMENT: should be set to the alignment of the CPU
         *    4 byte alignment -> #define MEM_ALIGNMENT 4
         *    2 byte alignment -> #define MEM_ALIGNMENT 2
         */
#define MEM_ALIGNMENT           4
//#define MEMP_OVERFLOW_CHECK             2

        /**
         * MEM_SIZE: the size of the heap memory. If the application will send
         * a lot of data that needs to be copied, this should be set high.
        */
//memory space of sending pbufs, whichis determined by MEM_SIZE constant.
 #define MEM_SIZE                    (70*1024)   // 16000 200*1024
 
#define MEMP_OVERFLOW_CHECK 1
//#define MEMP_SANITY_CHECK 0 
//#define MEM_USE_POOLS 0 
//#define MEMP_USE_CUSTOM_POOLS 0  

/*
   ------------------------------------------------
   ---------- Internal Memory Pool Sizes ----------
   ------------------------------------------------
*/
        /**
         * MEMP_NUM_PBUF: the number of memp struct pbufs (used for PBUF_ROM and PBUF_REF).
         * If the application sends a lot of data out of ROM (or other static memory),
         * this should be set high.
         */
//#define MEMP_NUM_PBUF           40

//#define MEMP_NUM_RAW_PCB 4

        /* MEMP_NUM_UDP_PCB: the number of UDP protocol control blocks. One
           per active UDP "connection". */
#define MEMP_NUM_UDP_PCB        4

        /**
         * MEMP_NUM_TCP_PCB: the number of simulatenously active TCP connections.
         * (requires the LWIP_TCP option)
         */
#define MEMP_NUM_TCP_PCB         4          //4// 4

//#define MEMP_NUM_REASSDATA 5     // ͬʱڶеȴװIPIPIPƬ

//#define MEMP_NUM_ARP_QUEUE 30 //

//#define MEMP_NUM_IGMP_GROUP 8 //

#define MEMP_NUM_NETBUF    4       // netbufsṹĿʹseq APIҪ

//#define MEMP_NUM_NETCONN 4     // netconnsṹĿʹseq APIҪ

//#define MEMP_NUM_TCPIP_MSG_API 20 // tcpip_msgṹĿ

//#define MEMP_NUM_TCPIP_MSG_INPKT 8 //հʱtcpip_msgṹĿ
//Should match the size of TCP_WND, see the compile message.
#define PBUF_POOL_SIZE                  63//16//43->100K;  76 ->150K; 63->130K
/*
   --------------------------------
   ---------- ARP options -------
   --------------------------------
*/
//#define LWIP_ARP 1 
//#define ARP_TABLE_SIZE              10
//#define ARP_QUEUEING                1

        /** ETH_PAD_SIZE: number of bytes added before the ethernet header to ensure
         * alignment of payload after that header. Since the header is 14 bytes long,
         * without this padding e.g. addresses in the IP header will not be aligned
         * on a 32-bit boundary, so setting this to 2 can speed up 32-bit-platforms.
         */
#define ETH_PAD_SIZE                2

#define ETHARP_SUPPORT_STATIC_ENTRIES   1

/*
   --------------------------------
   ---------- IP options ----------
   --------------------------------
*/
#define IP_FORWARD                  0
#define IP_FORWARD_ALLOW_TX_ON_RX_NETIF 0
#define IP_OPTIONS_ALLOWED    1
#define IP_REASSEMBLY                   0
#define IP_FRAG                         1
        /**
         * IP_REASS_MAXAGE: Maximum time (in multiples of IP_TMR_INTERVAL - so seconds, normally)
         * a fragmented IP packet waits for all fragments to arrive. If not all fragments arrived
         * in this time, the whole packet is discarded.
         */
#define IP_REASS_MAXAGE                 3

#define LWIP_RANDOMIZE_INITIAL_LOCAL_PORTS 1


/*
   ----------------------------------
   ---------- ICMP options ----------
   ----------------------------------
*/

/*
   ----------------------------------
   ---------- DHCP options ----------
   ----------------------------------
*/
#define LWIP_DHCP                       1

/*
   ------------------------------------
   ---------- AUTOIP options ----------
   ------------------------------------
*/
/*
   ----------------------------------
   ---------- SNMP options ----------
   ----------------------------------
*/
/*
   ----------------------------------
   ---------- IGMP options ----------
   ----------------------------------
*/
/*
   ----------------------------------
   ---------- DNS options -----------
   ----------------------------------
*/

/*
   ---------------------------------
   ---------- UDP options ----------
   ---------------------------------
*/
/*
   ---------------------------------
   ---------- TCP options ----------
   ---------------------------------
*/
#define TCP_QUEUE_OOSEQ                 1//(LWIP_TCP)
        /**
         * TCP_MSS: TCP Maximum segment size. (default is 536, a conservative default,
         * you might want to increase this.)
         * For the receive side, this MSS is advertised to the remote side
         * when opening a connection. For the transmit size, this MSS sets
         * an upper limit on the MSS advertised by the remote host.
         */
// The best, this value should be 1500. same as pc MTU.
#define TCP_MSS                        1500//1500   //2048   //536  

        /**
         * TCP_SND_BUF: TCP sender buffer space (bytes).
         * To achieve good performance, this should be at least 2 * TCP_MSS.
         */
//if this value is large, then the MEM_SIZE may not be enough. At same time,need more buf alloc.
//Larger, then recv/send rate faster. 
/*
snd_buf does limit the memory space of sending pbufs, not for the whole stack but for single connections. 
It is used to prevent one connection from using up all memory.
*/
#define TCP_SND_BUF                  (30* TCP_MSS)     //( 21,  43* TCP_MSS)
//       W        R
//12   2.65    3.45
// 16  2.57    3.35 

//20   2.69    3.45

// 22  2.59    3.32
//25   2.64    3.41
//30   2.50    3.35
// 40  2.51    3.42

        /**
         * TCP_WND: The size of a TCP window.  This must be at least 
         * (2 * TCP_MSS) for things to work well
         */
//Larger ,better. U16_t limit this value. set it close to 65535         
#define TCP_WND                        (43* TCP_MSS)      // (15* TCP_MSS)       // max : 15 
//#define TCP_MAXRTX                     12
//#define TCP_SYNMAXRTX                4

        /**
         * TCP_SND_QUEUELEN: TCP sender buffer space (pbufs). This must be at least
         * as much as (2 * TCP_SND_BUF/TCP_MSS) for things to work.
         */
//Consideration of stability, set it to default.         
//#define TCP_SND_QUEUELEN        ((4 * (TCP_SND_BUF) + (TCP_MSS - 1))/(TCP_MSS))
#define TCP_SND_QUEUELEN        63

        /**
         * MEMP_NUM_TCP_SEG: the number of simultaneously queued TCP segments.
         * (requires the LWIP_TCP option)
         */
#define MEMP_NUM_TCP_SEG       TCP_SND_QUEUELEN+1 //((5 * (TCP_SND_BUF) + (TCP_MSS - 1))/(TCP_MSS))//TCP_SND_QUEUELEN


/*
   ----------------------------------
   ---------- Pbuf options ----------
   ----------------------------------
*/



/*
   ------------------------------------------------
   ---------- Network Interfaces options ----------
   ------------------------------------------------
*/
        /*
         * LWIP_NETIF_HWADDRHINT==1: Cache link-layer-address hints (e.g. table indices)
        */
#define LWIP_NETIF_HWADDRHINT           1


/*
   ------------------------------------
   ---------- LOOPIF options ----------
   ------------------------------------
*/
/*
   ------------------------------------
   ---------- SLIPIF options ----------
   ------------------------------------
*/


/*
   ------------------------------------
   ---------- Thread options ----------
   ------------------------------------
*/
#define TCPIP_THREAD_NAME              "tcpip_thread"
#define TCPIP_THREAD_STACKSIZE     768

#define TCPIP_THREAD_PRIO               9
#define TCPIP_MBOX_SIZE                  100

#define DEFAULT_THREAD_STACKSIZE           256  
#define DEFAULT_RAW_RECVMBOX_SIZE       100
#define DEFAULT_UDP_RECVMBOX_SIZE       100
/**
 * DEFAULT_TCP_RECVMBOX_SIZE: The mailbox size for the incoming packets on a
 * NETCONN_TCP. The queue size value itself is platform-dependent, but is passed
 * to sys_mbox_new() when the recvmbox is created.
 */
#define DEFAULT_TCP_RECVMBOX_SIZE       100
#define DEFAULT_ACCEPTMBOX_SIZE         50
/*
   ----------------------------------------------
   ---------- Sequential layer options ----------
   ----------------------------------------------
*/
/*
   ------------------------------------
   ---------- Socket options ----------
   ------------------------------------
*/

#define LWIP_SO_RCVTIMEO                1

/*
   ----------------------------------------
   ---------- Statistics options ----------
   ----------------------------------------
*/
#define LWIP_STATS_DISPLAY              1
/*
   ---------------------------------
   ---------- PPP options ----------
   ---------------------------------
*/
/*
   --------------------------------------
   ---------- Checksum options ----------
   --------------------------------------
*/

/*
   ---------------------------------------
   ---------- Hook options ---------------
   ---------------------------------------
*/

/*
   ---------------------------------------
   ---------- Debugging options ----------
   ---------------------------------------
*/

        //* NETIF_DEBUG: Enable debugging in netif.c.
#define NETIF_DEBUG                     LWIP_DBG_OFF
        //* PBUF_DEBUG: Enable debugging in pbuf.c.
#define PBUF_DEBUG                      LWIP_DBG_OFF
        // * SOCKETS_DEBUG: Enable debugging in sockets.c.
#define SOCKETS_DEBUG                LWIP_DBG_OFF
#define API_LIB_DEBUG                  LWIP_DBG_OFF
#define API_MSG_DEBUG                LWIP_DBG_OFF
        //* ICMP_DEBUG: Enable debugging in icmp.c.
#define IGMP_DEBUG                      LWIP_DBG_OFF
        //* INET_DEBUG: Enable debugging in inet.c.
#define INET_DEBUG                       LWIP_DBG_OFF
        // * IP_REASS_DEBUG: Enable debugging in ip_frag.c for both frag & reass.
#define IP_REASS_DEBUG               LWIP_DBG_OFF
#define DHCP_DEBUG                     LWIP_DBG_OFF


#define UDP_DEBUG                        LWIP_DBG_OFF
#define TCP_DEBUG                        LWIP_DBG_OFF

/**
 * TCP_CWND_DEBUG: Enable debugging for TCP congestion window.
 */
#define TCP_CWND_DEBUG                  LWIP_DBG_OFF
/**
 * TCP_WND_DEBUG: Enable debugging in tcp_in.c for window updating.
 */
#define TCP_WND_DEBUG                   LWIP_DBG_OFF

/**
 * TCP_OUTPUT_DEBUG: Enable debugging in tcp_out.c output functions.
 */
#define TCP_OUTPUT_DEBUG                LWIP_DBG_OFF

/**
 * TCP_QLEN_DEBUG: Enable debugging for TCP queue lengths.
 */
#define TCP_QLEN_DEBUG                  LWIP_DBG_OFF





#define TCPIP_DEBUG                     LWIP_DBG_OFF
#define TCP_INPUT_DEBUG              LWIP_DBG_OFF

#define IP_DEBUG                           LWIP_DBG_OFF
#define RAW_DEBUG                       LWIP_DBG_OFF
#define ETHARP_DEBUG                   LWIP_DBG_OFF



/*
#define TCP_SND_QUEUELEN                40
#define MEMP_NUM_TCP_SEG                TCP_SND_QUEUELEN
#define TCP_SND_BUF                     (12 * TCP_MSS)
#define TCP_WND                         (10 * TCP_MSS)
*/




#endif /* __LWIPOPTS_H__ */
