
/*
	FreeRTOS.org V5.1.0 - copyright (C) 2003-2006 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************
	See http://www.FreeRTOS.org for documentation, latest information, license
	and contact details.  Please ensure to read the configuration and relevant
	port sections of the online documentation.
	***************************************************************************
*/

/*------------------------------------------------------------------------------*/
/*                            PROTOTYPES                                        */
/*------------------------------------------------------------------------------*/

/* Standard includes. */
#include <stdio.h>
#include <string.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

#include "stm32f10x.h"

/* Demo includes. */
#include "BasicWEB.h"

/* lwIP includes. */
#include "lwip/tcpip.h"
/*   included in tcpip.h
#include "lwip/ip_addr.h"
#include "lwip/err.h"
#include "lwip/igmp.h"
#include "lwip/opt.h"
#include "lwip/api_msg.h"
#include "lwip/netifapi.h"
#include "lwip/pbuf.h"
#include "lwip/api.h"
#include "lwip/sys.h"
#include "lwip/timer.h"
#include "lwip/netif.h"
*/

//#include "lwip/memp.h"
//#include "lwip/stats.h"
//#include "netif/loopif.h"
#include "lwip/sockets.h"
#include "lwip/tcp.h"
//#include <errno.h>
//#include "stm32f10x_it.h"
//#include "para_config.h"

#include "port/wifi_user_interface.h"

   //DongYoucai
#include "lwip/dhcp.h"
#include "esp_debug.h"


static void vBasicWEBServer( void *pvParameters );
static void vSendTaskFunc( void *pvParameters );

/*------------------------------------------------------------------------------*/
/*                            GLOBALS                                          */
/*------------------------------------------------------------------------------*/
xQueueHandle  EglWifiUserMbox;

static struct netif EMAC_if;
static int remotefd;

typedef enum STATION_STATE {
        STA_INIT = 0,
        STA_BOOTUP,
        STA_DISCON,
        STA_CON,
        //STA_STOP
} STATION_STATE;
STATION_STATE EMACStaSt;

typedef enum SOCKET_STATE {
        SOCKET_INIT = 0,
        SOCKET_CLOSE,
        SOCKET_OP,
        //WIFI_STOP
} SOCKET_STATE;
SOCKET_STATE DevSocketSt;

#if 0

extern unsigned char g_ar6k_tx_inactive;
	
extern u8 g_ep1_status;
extern u8 MobState, gMobStateChanged;
extern u8 g_power_level;


#define MAX_WAIT_TIMEOUT		10000
#define DEFAULT_SYNC_PERIOD     1000

unsigned long g_sync_time = DEFAULT_SYNC_PERIOD;


void vBasicWEBServer( void *pvParameters );
void vSendTaskFunc( void *pvParameters );

/*------------------------------------------------------------------------------*/
/*                            GLOBALS                                          */
/*------------------------------------------------------------------------------*/
static struct netif EMAC_if;

/*------------------------------------------------------------------------------*/
/*                            FUNCTIONS                                         */
/*------------------------------------------------------------------------------*/
static int remotefd;

static void
lwip_sanity_check(void)
{
  /* Warnings */
#if LWIP_NETCONN
  if (MEMP_NUM_NETCONN > (MEMP_NUM_TCP_PCB+MEMP_NUM_TCP_PCB_LISTEN+MEMP_NUM_UDP_PCB+MEMP_NUM_RAW_PCB))
    LWIP_PLATFORM_DIAG(("lwip_sanity_check: WARNING: MEMP_NUM_NETCONN should be less than the sum of MEMP_NUM_{TCP,RAW,UDP}_PCB+MEMP_NUM_TCP_PCB_LISTEN\n"));
#endif /* LWIP_NETCONN */

#if LWIP_TCP
  if (MEMP_NUM_TCP_SEG < TCP_SND_QUEUELEN)
    LWIP_PLATFORM_DIAG(("lwip_sanity_check: WARNING: MEMP_NUM_TCP_SEG should be at least as big as TCP_SND_QUEUELEN\n"));
  if (TCP_SND_QUEUELEN < (2 * (TCP_SND_BUF/TCP_MSS)))
    LWIP_PLATFORM_DIAG(("lwip_sanity_check: WARNING: TCP_SND_QUEUELEN must be at least as much as (2 * TCP_SND_BUF/TCP_MSS) for things to work\n"));
  if (TCP_SNDLOWAT > TCP_SND_BUF)
    LWIP_PLATFORM_DIAG(("lwip_sanity_check: WARNING: TCP_SNDLOWAT must be less than or equal to TCP_SND_BUF.\n"));
  if (TCP_WND > (PBUF_POOL_SIZE*PBUF_POOL_BUFSIZE))
    LWIP_PLATFORM_DIAG(("lwip_sanity_check: WARNING: TCP_WND is larger than space provided by PBUF_POOL_SIZE*PBUF_POOL_BUFSIZE\n"));
  if (TCP_WND < TCP_MSS)
    LWIP_PLATFORM_DIAG(("lwip_sanity_check: WARNING: TCP_WND is smaller than MSS\n"));
#endif /* LWIP_TCP */

}
#endif


void vlwIPInit( void )
{
/* Initialize lwIP and its interface layer. */
#if 0
       lwip_sanity_check();
	sys_init();
	mem_init();
	memp_init();
	pbuf_init();
	netif_init();
	ip_init();

	sys_set_state(( signed portCHAR * ) "lwIP",256 );	//256
	tcpip_init( NULL, NULL );	
	sys_set_default_state();
	
       sys_set_state(( signed portCHAR * ) "MainTask", 384);	//384
	sys_thread_new( vBasicWEBServer, ( void * ) NULL,  basicwebWEBSERVER_PRIORITY );
	sys_set_default_state();
    
       sys_set_state(( signed portCHAR * ) "SendTask", 128);
	sys_thread_new( vSendTaskFunc, ( void * ) NULL, mainSEND_TASK_PRIORITY ); 
	sys_set_default_state();
#endif
       EMACStaSt = STA_INIT;
       DevSocketSt = SOCKET_INIT;
       tcpip_init(NULL,NULL);
	xTaskCreate( vBasicWEBServer, ( signed char * ) "MainTask", 384, NULL, 5, NULL );
       xTaskCreate( vSendTaskFunc, ( signed char * ) "SendTask",256, NULL, 5, NULL );
}

#if 0



char GetIP = 0;

//DongYoucai  Dhcp_client thread 

Uint32 dhcp_client_timer(Uint32 interval, void *arg)
{

	static int mscnt = 0;
	(void *) arg;
	if(GetIP == 0)
		dhcp_fine_tmr();
	mscnt += DHCP_FINE_TIMER_MSECS;
	if (mscnt >= DHCP_COARSE_TIMER_SECS*1000)
	{
		dhcp_coarse_tmr();
		mscnt = 0;
	}
}

//DongYoucai  Dhcp_client thread 
void dhcp_client_thread(void *arg)
{
	(void *) arg;
	do
	{	
		igmp_tmr();
		vTaskDelay(100);	//fine_tmr is 500ms. coarse_tmr is 60s
	}while(1);

}
#endif

//unsigned long sync_ticks;
#define RCV_BUF_LEN (1024)

void vLWIPMainTask()
{
	int listenfd; 
	unsigned char *buf;

	fd_set readset/*,exceptset*/;
	int rev_buf_start = 0;

	//if(MobState == MOB_MODE_USB) return;//xinghua 20110712

 	printf("Start LWIP Task, ");

	listenfd = socket(AF_INET,SOCK_STREAM,0);
	
	{
		struct sockaddr_in local_addr;  

		local_addr.sin_family = AF_INET;
		local_addr.sin_port = htons(8888);
		local_addr.sin_len = sizeof(local_addr);
		//local_addr.sin_addr.s_addr = INADDR_ANY;
		local_addr.sin_addr.s_addr = EMAC_if.ip_addr.addr;

		if (bind(listenfd, (struct sockaddr *) &local_addr, sizeof(local_addr)) < 0) 
		{
			return ;
		}
		if (listen(listenfd, 1) == -1)
		{
			return;
		}
	}	
	printf("Listening...\n");

	buf = (u8 *)pvPortMalloc_1(RCV_BUF_LEN);
	while(1)
	{
                {
                        struct sockaddr_in remote_addr;
                        int len = sizeof(remote_addr);
                        printf("remotefd waiting \n");
                        remotefd = accept(listenfd, (struct sockaddr *)&remote_addr, &len);
                }

                //if(MobState == MOB_MODE_USB)
                //break;

                {
                        int optval=1;
                        setsockopt(remotefd, IPPROTO_TCP, TCP_NODELAY, &optval, sizeof(optval));	
                        //int opt = 100;
                       // lwip_setsockopt(remotefd, SOL_SOCKET, SO_RCVTIMEO, &opt, sizeof(int));
                }

                if(remotefd != -1)
                {
                        //MobState = MOB_WIFI_CONNECTED;
                        DevSocketSt = SOCKET_OP;
                        //gMobStateChanged = 1;
                        //printf("-Socket Accept-remotefd:%d,MobState:%d",remotefd,MobState);
                        printf("-Socket Accept-remotefd:%d",remotefd);

                        //last_ticks = xTaskGetTickCount();
                        int ret;
                        rev_buf_start = 0;
                        memset(buf,0,RCV_BUF_LEN);

                        for(;;)
                        {				
                                struct timeval timeout;
                                timeout.tv_sec = 0;
                                timeout.tv_usec = 5000;
//printf("-");
                                FD_ZERO(&readset);
                                FD_SET(remotefd, &readset);

                                ret = lwip_select(remotefd+1, &readset,0, 0, &timeout);
                                if(ret > 0)
                                {
                                        if (FD_ISSET(remotefd, &readset))
                                        {
                                                ret = recv(remotefd,buf,RCV_BUF_LEN,/*MSG_DONTWAIT*/0);
                                                if(ret <= 0)
                                                {
                                                        //printf("-Socket Close (%d,%d)-",ret, errno);	
                                                        printf("-Socket Close (%d)-",ret);	
                                                        close(remotefd);
                                                        DevSocketSt = SOCKET_CLOSE;
                                                        remotefd = -1;
                                                        break;
                                                }
                                                else if (ret > 0)
                                                {
                                                        //printf("r_len:%d\n",ret);
                                                        //show_buf(buf, RCV_BUF_LEN);
                                                }
                                        }
                                        // printf("r\n");
                                }	
                                else if(ret == 0)//timeout
                                {
                                        //printf("-");
                                }
                                else if(ret < 0)//error
                                {
                                        printf("\n--Socket error--");	//never goes here
                                }  
                        }
                }
              /*         
    		if(MobState == MOB_MODE_USB)
    			break;
    		else
    		{
    			gMobStateChanged = 1;
    			MobState = MOB_MODE_WIFI;	
    		}
            
    		g_ar6k_tx_inactive = 0;
    		*/
    		vTaskDelay(1);
	}
	vPortFree(buf);
}

void vBasicWEBServer( void *pvParameters )
{
printf("BasicWEBServer created\n");
	extern err_t ethernetif_init( struct netif *netif );

       /* Parameters are not used - suppress compiler error. */
        ( void ) pvParameters;
       
        UserAddMbox( (void*)&EglWifiUserMbox,40 );
        xTaskCreate( vCheckTask, ( signed portCHAR * ) "Wifi Evt Mbox Check", 256, NULL, 2, NULL );

#if 0		
        //DongYoucai AP server initialization
        if ( ProjWifiNetworkPara.WifiMode == 2 ) 
        {
            struct ip_addr xIpAddr, xNetMast, xGateway;
            /* Create and configure the EMAC interface. */
            IP4_ADDR( &xIpAddr, emacIPADDR0, emacIPADDR1, emacIPADDR2, emacIPADDR3 );
            IP4_ADDR( &xNetMast, emacNET_MASK0, emacNET_MASK1, emacNET_MASK2, emacNET_MASK3 );
            IP4_ADDR( &xGateway, emacGATEWAY_ADDR0, emacGATEWAY_ADDR1, emacGATEWAY_ADDR2, emacGATEWAY_ADDR3 );
            netif_add( &EMAC_if, &xIpAddr, &xNetMast, &xGateway, NULL, ethernetif_init, tcpip_input );
            /* make it the default interface */
            netif_set_default( &EMAC_if );
            /* bring it up */
            netif_set_up(&EMAC_if);

            while( WifiModeStatus != MOB_WIFI_AP_IDLE )
            vTaskDelay(10);

            vLWIPMainTask();
        }
#endif
	//DongYoucai  add dhcp start here.
	//if ( ProjWifiNetworkPara.WifiMode == 1 )            //Sta
       if ( 1 )
	{
		struct ip_addr xIpAddr, xNetMast, xGateway;

		IP4_ADDR( &xIpAddr, 0, 0, 0, 0 );
		IP4_ADDR( &xNetMast, 1, 1, 1, 1 );
		IP4_ADDR( &xGateway, 2, 2, 2, 2 );
		
		netif_add( &EMAC_if, &xIpAddr, &xNetMast, &xGateway, NULL, ethernetif_init,  tcpip_input );

		// make it the default interface 
		netif_set_default( &EMAC_if );
        
              while( EMACStaSt != STA_BOOTUP )
			vTaskDelay(100);
              
            UserConToAP();

		while( EMACStaSt != STA_CON )
			vTaskDelay(100);

		int ret = dhcp_start(&EMAC_if);	
		if (ret == ERR_OK)
		{
			vTaskDelay(100);
			//t = SDL_AddTimer(500,dhcp_client_timer ,NULL);	
              }
        
		do
		{	
			if( EMAC_if.dhcp != NULL && EMAC_if.dhcp->state == DHCP_BOUND )
			{

				dhcp_stop(&EMAC_if);   //Can't stop dhcp_client, for renew the IP after rent time is over.
				printf("get IP:\n");
				//WifiModeStatus = MOB_WIFI_STA_IDLE;
				//GetIP = 1;
				break;
			}
			vTaskDelay(100);	//fine_tmr is 500ms. coarse_tmr is 60s
		}while(1);
	  #if 1
		vLWIPMainTask();
        #endif
                while(1)
                    vTaskDelay(100);
        }

	/* Nothing else to do.  No point hanging around. */
	vTaskDelete( NULL );
}

void vSendTaskFunc( void *pvParameters )
{

printf("Send thread  ok \n");
	int ret;
	fd_set writeset;
	unsigned long ticks, last_ticks;

	struct timeval timeout;
	timeout.tv_sec = 0;
	timeout.tv_usec = 5000;

 	//printf("Start Send Task, ");

	last_ticks = xTaskGetTickCount();
	while (1)
	{
#if 1
		if (EMACStaSt != STA_CON)
		{
			//printf(".");
			vTaskDelay(50);
			continue;
		}

		/*IMPORTANT NOTES: we should send sync flag here instead of in Receive Task. 
		Because Function call may block receive task thread, so that sync flag may not be sent whithin 1 sec period.
		*/
		ticks = xTaskGetTickCount();//xinghua	
		
		//if((ticks - last_ticks) > g_sync_time  && (MobState == MOB_WIFI_CONNECTED))
              if((ticks - last_ticks) > 1000  && (DevSocketSt == SOCKET_OP))
		{
			last_ticks = ticks;
			FD_ZERO(&writeset); 
			FD_SET(remotefd, &writeset);

			ret = lwip_select(remotefd+1, 0, &writeset, 0, &timeout);
printf("\n ret: %d ",ret);

			if (ret > 0 && FD_ISSET(remotefd, &writeset))
			{
				char p[3];
				//printf("\n-S-",remotefd);

				p[0] = 0x5a;
				p[1] = 1;
				//p[2] = g_power_level;
                            p[2] = 0xff;
				if (send(remotefd, (void *)p, 3,0) < 3)
					;//printf("X");
				else
					;//printf("T");
			}
		}
#endif
            vTaskDelay(100);
	}

	vTaskDelete( NULL );
}


/*________________Below are for wifi evt mbox check___________________*/

//#include "esp_debug.h"
//xQueueHandle  EglWifiUserMbox;
struct egl_wifi_user_msg 
{
        u8 id;
        u16 len;
        u8 *data;
}__packed; 

static void vCheckTask( void *pvParameters )
{
        //UserAddMbox( (void*)&EglWifiUserMbox,50 );
        
        struct egl_wifi_user_msg *msg;
        for( ;; )
        {
               if( pdTRUE == xQueueReceive( EglWifiUserMbox, &(msg), 0 ) )
               {
                        printf("msg id: %d\n",msg->id);
                        show_buf(msg->data, msg->len);
                        
                        switch(msg->id)
                        {
                                case 1:
                                        break;
                                case 2:
                                        break;   
                                case 3:
                                        EMACStaSt = STA_CON;
                                        break;
                                case 4:
                                        EMACStaSt = STA_DISCON;
                                        break;
                                case 10:
                                        break;
                                case 12:
                                        EMACStaSt = STA_BOOTUP;
                                        memcpy(EMAC_if.hwaddr,msg->data,6);
                                        break;
                                 default:
                                        break;
                         }
                        
                        vPortFree_1(msg->data);
                        vPortFree_1(msg);
               }
               vTaskDelay(100);
        }
}

/*-----------------------------------------------------------*/
